const express = require('express');
const bodyParser = require('body-parser');
const session = require('express-session');
const path = require('path');
const fs = require('fs');
const bcrypt = require('bcryptjs');
const multer = require('multer');
const flash = require('connect-flash');
const mime = require('mime-types');
const sharp = require('sharp');

const app = express();

// Middleware
app.use(bodyParser.urlencoded({ extended: true, limit: '10mb' }));
app.use(bodyParser.json({ limit: '10mb' }));
app.use(express.static(path.join(__dirname, 'public')));
app.set('view engine', 'ejs');
// Tell Express to trust the X-Forwarded-For header
app.set('trust proxy', true);

// Session middleware
app.use(session({
    secret: 'rahasia123',
    resave: false,
    saveUninitialized: true,
    cookie: { 
        secure: false, // set to true if using https
        maxAge: 24 * 60 * 60 * 1000 // 24 jam
    }
}));

// Flash middleware
app.use(flash());

// Middleware untuk menambahkan data user dan flash messages ke response
app.use((req, res, next) => {
    // Tambahkan user data dan flash messages ke res.locals
    // sehingga tersedia di semua template tanpa perlu meneruskan secara manual
    res.locals.user = req.session.user || null;
    res.locals.userRole = req.session.userRole || null;
    res.locals.success_msg = req.flash('success_msg');
    res.locals.error_msg = req.flash('error_msg');
    res.locals.isAuthenticated = req.session.isAuthenticated || false;
    next();
});

// Menyediakan pengaturan tema untuk semua rute admin
app.use('/admin', (req, res, next) => {
    console.log("test");
    if (!req.session.user) {
        return next();
    }
    
    try {
        // Get user data including settings
        let users = [];
        if (fs.existsSync(usersPath)) {
            users = JSON.parse(fs.readFileSync(usersPath));
        }
        
        const currentUser = users.find(user => user.username === req.session.user.username);
        
        // Tambahkan data user lengkap ke res.locals
        if (currentUser) {
            res.locals.userData = {
                ...req.session.user,
                ...currentUser
            };
            res.locals.userRole = req.session.userRole || currentUser.role || 'user';
            
            // Add settings to res.locals for access in all templates
            if (currentUser.settings) {
            res.locals.userSettings = currentUser.settings;
        } else {
            // Default settings if not set
            res.locals.userSettings = {
                theme: 'light',
                fontSize: 'medium',
                themeColor: 'blue',
                compactSidebar: false
            };
        }
        } else {
            res.locals.userData = req.session.user;
            res.locals.userRole = req.session.userRole || 'user';
            
            // Default settings if user not found
            res.locals.userSettings = {
                theme: 'light',
                fontSize: 'medium',
                themeColor: 'blue',
                compactSidebar: false
            };
        }
        
        next();
    } catch (error) {
        console.error('Error setting user data middleware:', error);
        // Continue without settings if error occurs
        next();
    }
});

// Cek file JSON ada atau tidak
const dataPath = path.join(__dirname, 'data');
const postsPath = path.join(dataPath, 'posts.json');
const usersPath = path.join(dataPath, 'users.json');
const categoriesPath = path.join(dataPath, 'categories.json');
const commentsPath = path.join(dataPath, 'comments.json');
const mediaPath = path.join(dataPath, 'media.json');
const settingsPath = path.join(dataPath, 'settings.json');
const uploadsPath = path.join(__dirname, 'public', 'uploads');

// Memastikan folder data dan uploads ada
if (!fs.existsSync(dataPath)) {
    fs.mkdirSync(dataPath, { recursive: true });
}

if (!fs.existsSync(uploadsPath)) {
    fs.mkdirSync(uploadsPath, { recursive: true });
}

// Buat file JSON jika belum ada
if (!fs.existsSync(postsPath)) {
    fs.writeFileSync(postsPath, JSON.stringify([]));
}

if (!fs.existsSync(usersPath)) {
    fs.writeFileSync(usersPath, JSON.stringify([{
        username: 'admin',
        password: '$2a$10$XFE/UQEXfwOxHR.AfK8zeeHFVMKn1RqF9hqOCj9Z.qYhvwxvL5Lxe' // password: admin123
    }]));
}

if (!fs.existsSync(categoriesPath)) {
    fs.writeFileSync(categoriesPath, JSON.stringify([]));
}

if (!fs.existsSync(commentsPath)) {
    fs.writeFileSync(commentsPath, JSON.stringify([]));
}

if (!fs.existsSync(mediaPath)) {
    fs.writeFileSync(mediaPath, JSON.stringify([]));
}

if (!fs.existsSync(settingsPath)) {
    fs.writeFileSync(settingsPath, JSON.stringify({
        general: {
            siteName: "Nama Website Anda",
            siteDescription: "Deskripsi singkat tentang website Anda.",
            siteUrl: "https://www.website-anda.com",
            adminEmail: "admin@website-anda.com",
            timezone: "Asia/Jakarta"
        },
        content: {
            postsPerPage: 10,
            defaultCategory: 1,
            enableComments: true,
            moderateComments: true
        },
        design: {
            logoUrl: "https://via.placeholder.com/150x50",
            faviconUrl: "https://via.placeholder.com/32",
            primaryColor: "#0061ff",
            accentColor: "#60efff",
            fontFamily: "segoe-ui"
        },
        seo: {
            metaTitle: "Nama Website Anda - Tagline Anda",
            metaDescription: "Deskripsi website Anda untuk mesin pencari.",
            metaKeywords: "keyword1, keyword2, keyword3",
            metaRobots: "index,follow",
            enableSitemap: true,
            enableRobots: true
        },
        social: {
            facebookUrl: "",
            twitterUrl: "",
            instagramUrl: "",
            youtubeUrl: "",
            enableSocialSharing: true
        }
    }, null, 2), 'utf8');
}

// Setup multer untuk handling file uploads
const storage = multer.diskStorage({
    destination: function (req, file, cb) {
        // Pastikan direktori uploads ada
        if (!fs.existsSync(uploadsPath)) {
            console.log('Creating uploads directory');
            fs.mkdirSync(uploadsPath, { recursive: true });
        }
        cb(null, uploadsPath);
    },
    filename: function (req, file, cb) {
        const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
        // Simpan file tanpa ekstensi karena kita akan memproses dan menyimpan sebagai .jpg
        cb(null, 'img-' + uniqueSuffix);
    }
});

// Fungsi untuk memproses dan resize gambar
async function processImage(filePath, targetPath, options = {}) {
    try {
        console.log(`Processing image: ${filePath} to ${targetPath}`);
        
        // Default options
        const width = options.width || 1600;
        const height = options.height || 900;
        const format = options.format || 'jpeg';
        const quality = options.quality || 85;
        
        // Process with Sharp
        let sharpInstance = sharp(filePath)
            .resize(width, height, {
                fit: sharp.fit.inside,
                withoutEnlargement: true
            });
        
        // Apply format
        if (format === 'jpeg' || format === 'jpg') {
            sharpInstance = sharpInstance.toFormat('jpeg').jpeg({ quality });
        } else if (format === 'png') {
            sharpInstance = sharpInstance.toFormat('png').png({ quality });
        } else if (format === 'webp') {
            sharpInstance = sharpInstance.toFormat('webp').webp({ quality });
        } else {
            // Default to jpeg
            sharpInstance = sharpInstance.toFormat('jpeg').jpeg({ quality });
        }
        
        // Save to file
        await sharpInstance.toFile(targetPath);
        
        // Hapus file asli setelah diproses
        if (filePath !== targetPath) {
            fs.unlinkSync(filePath);
        }
        
        console.log(`Successfully processed image: ${targetPath}`);
        return true;
    } catch (error) {
        console.error('Error processing image:', error);
        // Try to save the original file by copying it instead
        try {
            if (filePath !== targetPath) {
                fs.copyFileSync(filePath, targetPath);
                console.log(`Fallback: Copied original file to ${targetPath}`);
                return true;
            }
        } catch (copyError) {
            console.error('Error even with fallback copy:', copyError);
        }
        return false;
    }
}

const upload = multer({ 
    storage: storage,
    limits: { fileSize: 10 * 1024 * 1024 }, // Limit to 10MB
    fileFilter: function (req, file, cb) {
        console.log(`Received file: ${file.originalname}, mimetype: ${file.mimetype}`);
        const allowedTypes = ['image/jpeg', 'image/png', 'image/gif', 'image/webp', 'image/jpg'];
        if (!allowedTypes.includes(file.mimetype)) {
            const error = new Error(`Format file tidak didukung: ${file.mimetype}. Hanya JPG, PNG, GIF, atau WebP.`);
            error.code = 'LIMIT_FILE_TYPE';
            return cb(error, false);
        }
        cb(null, true);
    }
});

// Middleware untuk menangani error multer
app.use((err, req, res, next) => {
    if (err instanceof multer.MulterError) {
        console.error('Multer error:', err);
        if (err.code === 'LIMIT_FILE_SIZE') {
            return res.status(400).json({ 
                success: false, 
                message: 'Ukuran file terlalu besar. Maksimal 10MB.' 
            });
        }
        return res.status(400).json({ 
            success: false, 
            message: `Error upload: ${err.message}` 
        });
    } else if (err.code === 'LIMIT_FILE_TYPE') {
        console.error('File type error:', err);
        return res.status(400).json({ 
            success: false, 
            message: err.message || 'Format file tidak didukung' 
        });
    } else if (err) {
        console.error('Other error in middleware:', err);
        return res.status(500).json({ 
            success: false, 
            message: 'Terjadi kesalahan saat mengunggah file' 
        });
    }
    next();
});

// Helper function untuk generate ID
function generateId() {
    return Math.random().toString(36).substring(2, 15) + Math.random().toString(36).substring(2, 15);
}

// Fungsi untuk menghasilkan slug dari judul
function generateSlug(text) {
    return text.toString().toLowerCase()
        .replace(/\s+/g, '-')           // Replace spaces with -
        .replace(/[^\w\-]+/g, '')       // Remove all non-word chars
        .replace(/\-\-+/g, '-')         // Replace multiple - with single -
        .replace(/^-+/, '')             // Trim - from start of text
        .replace(/-+$/, '')             // Trim - from end of text
        .replace(/[^a-z0-9-]/g, '');    // Keep only lowercase letters, numbers, and hyphens
}

// Middleware untuk mengecek autentikasi
const checkAuth = (req, res, next) => {
    if (req.session.isAuthenticated) {
        return next();
    }
    
    req.session.errorMessage = 'Silakan login terlebih dahulu.';
    res.redirect('/admin/login');
};

// Middleware untuk mengecek role
const checkRole = (roles) => {
    return (req, res, next) => {
        if (!req.session.isAuthenticated) {
            return res.redirect('/admin/login');
        }
        
        if (roles.includes(req.session.userRole)) {
            next();
        } else {
            req.flash('error_msg', 'Anda tidak memiliki akses ke halaman ini');
            return res.redirect('/admin/dashboard');
        }
    };
};

// Import middleware IP whitelist
const checkIpWhitelist = require('./ipWhitelist');

// Terapkan middleware IP whitelist untuk semua rute admin
app.use('/admin', checkIpWhitelist);

// Impor dan gunakan router profil
const profileRoutes = require('./profile-routes');
app.use('/admin', profileRoutes);

// Variabel global untuk menyimpan percobaan login yang gagal
const failedLoginAttempts = {};

// Admin routes
app.get('/admin/login', checkIpWhitelist, (req, res) => {
    if (req.session.isAuthenticated) {
        return res.redirect('/admin/dashboard');
    }
    
    // Mengambil pesan error dari session jika ada (tanpa menggunakan req.flash)
    const errorMessage = req.session.errorMessage;
    
    // Hapus pesan error dari session setelah diambil
    if (req.session.errorMessage) {
        delete req.session.errorMessage;
        req.session.save();
    }
    
    console.log('Rendering login dengan error message:', errorMessage);
    
    res.render('admin/login', { 
        error: errorMessage || '',
        error_msg: ''
    });
});

app.post('/admin/login', (req, res) => {
    try {
        const { username, password } = req.body;
        console.log('Login attempt:', username);

        // Baca file users.json
        const usersData = fs.readFileSync(usersPath, 'utf8');
        const users = JSON.parse(usersData);
        console.log('Users loaded:', users.length);
        
        const user = users.find(u => u.username === username);
        
        // Check if user exists and if the account is locked
        if (user && user.status === 'locked') {
            console.log('User account locked:', username);
            req.session.errorMessage = 'Akun Anda terkunci karena terlalu banyak percobaan login yang gagal. Silakan hubungi administrator.';
            return res.redirect('/admin/login');
        }
        
        // Debug password checking
        if (user) {
            console.log('User found, checking password...');
            const passwordMatch = bcrypt.compareSync(password, user.password);
            console.log('Password match:', passwordMatch);
            
            if (passwordMatch) {
                // Reset failed login attempts for this user
                if (failedLoginAttempts[username]) {
                    delete failedLoginAttempts[username];
                }
                
                // Cek status user
                if (user.status === 'inactive') {
                    console.log('User inactive:', username);
                    req.session.errorMessage = 'Akun Anda tidak aktif. Silakan hubungi administrator.';
                    return res.redirect('/admin/login');
                }
                
                // Set session
                req.session.isAuthenticated = true;
                req.session.user = { 
                    username: user.username,
                    email: user.email || '',
                    displayName: user.displayName || user.username
                };
                // Simpan role user dalam session
                req.session.userRole = user.role || 'admin';
                console.log('Login berhasil untuk:', username, 'dengan role:', req.session.userRole);
                
                // Set flash message sukses
                req.flash('success_msg', 'Login berhasil! Selamat datang di panel admin.');
                return res.redirect('/admin/dashboard');
            } else {
                // Increment failed login attempts
                if (!failedLoginAttempts[username]) {
                    failedLoginAttempts[username] = 1;
                } else {
                    failedLoginAttempts[username]++;
                }
                
                console.log(`Failed login attempts for ${username}: ${failedLoginAttempts[username]}`);
                
                // If 3 or more failed attempts, lock the account
                if (failedLoginAttempts[username] >= 3) {
                    // Update user status to locked
                    const userIndex = users.findIndex(u => u.username === username);
                    if (userIndex !== -1) {
                        users[userIndex].status = 'locked';
                        fs.writeFileSync(usersPath, JSON.stringify(users, null, 4));
                        
                        // Reset failed login attempts
                        delete failedLoginAttempts[username];
                        
                        console.log(`Account locked for ${username} due to too many failed login attempts`);
                        req.session.errorMessage = 'Akun Anda telah terkunci karena terlalu banyak percobaan login yang gagal. Silakan hubungi administrator.';
                        return res.redirect('/admin/login');
                    }
                } else {
                    const attemptsLeft = 3 - failedLoginAttempts[username];
                    req.session.errorMessage = `Username atau password salah! Anda memiliki ${attemptsLeft} kesempatan lagi sebelum akun terkunci.`;
                }
            }
        } else {
            // For non-existent users, still show similar message for security
            req.session.errorMessage = 'Username atau password salah! Silakan coba lagi.';
        }
        
        console.log('Login gagal untuk:', username);
        
        req.session.save(() => {
            console.log('Error message saved to session:', req.session.errorMessage);
            return res.redirect('/admin/login');
        });
    } catch (error) {
        console.error('Error pada login:', error);
        req.session.errorMessage = 'Terjadi kesalahan sistem. Coba lagi nanti.';
        req.session.save(() => {
            return res.redirect('/admin/login');
        });
    }
});

app.get('/admin/logout', (req, res) => {
    req.session.destroy();
    res.redirect('/admin/login');
});

app.get('/admin/dashboard', checkAuth, (req, res) => {
    let posts = [];
    let categories = [];
    let comments = [];
    
    try {
        posts = JSON.parse(fs.readFileSync(postsPath));
        categories = JSON.parse(fs.readFileSync(categoriesPath));
        comments = JSON.parse(fs.readFileSync(commentsPath));
        
        // Sort posts by date (newest first)
        posts.sort((a, b) => {
            const dateA = new Date(a.date || a.createdAt);
            const dateB = new Date(b.date || b.createdAt);
            return dateB - dateA;
        });
        
        // Limit to 5 most recent posts for dashboard
        const recentPosts = posts.slice(0, 5);
        
        res.render('admin/dashboard', { 
            posts: recentPosts, 
            allPostsCount: posts.length,
            categories,
            commentsCount: comments.length,
            user: req.session.user,
            userRole: req.session.userRole,
            success_msg: req.flash('success_msg'),
            error_msg: req.flash('error_msg')
        });
        
    } catch (error) {
        console.error('Error reading data for dashboard:', error);
        res.render('admin/dashboard', { 
            posts: [], 
            allPostsCount: 0,
            categories: [],
            commentsCount: 0
        });
    }
});

// Posts management routes
app.get('/admin/posts', checkAuth, (req, res) => {
    try {
        let posts = JSON.parse(fs.readFileSync(postsPath, 'utf8'));
        
        // Sort by date (newest first)
        posts.sort((a, b) => {
            const dateA = new Date(a.date || a.createdAt);
            const dateB = new Date(b.date || b.createdAt);
            return dateB - dateA;
        });
        
        // Jika user adalah penulis (author), hanya tampilkan artikel yang mereka tulis
        if (req.session.userRole === 'author') {
            posts = posts.filter(post => post.author === req.session.user.username);
        }
        
        // Filter posts berdasarkan pencarian jika ada
        const searchQuery = req.query.search || '';
        if (searchQuery) {
            const searchTerms = searchQuery.toLowerCase().split(' ');
            posts = posts.filter(post => {
                const title = post.title.toLowerCase();
                const category = (post.category || '').toLowerCase();
                return searchTerms.some(term => 
                    title.includes(term) || category.includes(term)
                );
            });
        }
        
        // Pagination
        const page = parseInt(req.query.page) || 1;
        const postsPerPage = 10; // Jumlah post per halaman admin
        const totalPosts = posts.length;
        const totalPages = Math.ceil(totalPosts / postsPerPage);
        
        const startIndex = (page - 1) * postsPerPage;
        const endIndex = startIndex + postsPerPage;
        const paginatedPosts = posts.slice(startIndex, endIndex);
        
        res.render('admin/posts', { 
            posts: paginatedPosts, 
            success_msg: req.flash('success_msg'),
            error_msg: req.flash('error_msg'),
            currentPage: page,
            totalPages,
            totalPosts,
            searchQuery
        });
    } catch (error) {
        console.error('Error loading posts:', error);
        req.flash('error_msg', 'Error loading posts');
        res.redirect('/admin/dashboard');
    }
});

app.get('/admin/posts/new', checkAuth, checkRole(['admin', 'editor', 'author']), (req, res) => {
    const categories = JSON.parse(fs.readFileSync(categoriesPath));
    res.render('admin/post-form', { 
        post: { title: '', content: '', category: '' },
        categories,
        formAction: '/admin/posts/create',
        formTitle: 'Tambah Artikel Baru'
    });
});

app.post('/admin/posts/create', checkAuth, checkRole(['admin', 'editor', 'author']), upload.single('featuredImage'), async (req, res) => {
    const { title, content, category } = req.body;
    
    try {
        if (!title || !content) {
            req.flash('error_msg', 'Judul dan konten wajib diisi');
            return res.redirect('/admin/posts/new');
        }
        
        const newPost = {
            id: generateId(),
            title,
            content,
            category: category || 'Umum',
            author: req.session.user.username,
            authorId: req.session.user.id,
            date: new Date().toISOString(),
            lastUpdated: null,
            featuredImage: '',
            slug: generateSlug(title)
        };
        
        // Proses featured image jika ada
        if (req.file) {
            const originalPath = path.join(uploadsPath, req.file.filename);
            const targetFilename = req.file.filename + '.jpg';
            const targetPath = path.join(uploadsPath, targetFilename);
            
            const success = await processImage(originalPath, targetPath);
            if (success) {
                newPost.featuredImage = `/uploads/${targetFilename}`;
                
                // Add to media.json for media manager
                try {
                    let mediaData = [];
                    if (fs.existsSync(mediaPath)) {
                        mediaData = JSON.parse(fs.readFileSync(mediaPath, 'utf8'));
                    }
                    
                    // Add the new file
                    const fileStats = fs.statSync(targetPath);
                    mediaData.unshift({
                        id: generateId(),
                        filename: targetFilename,
                        originalname: req.file.originalname || targetFilename,
                        path: `/uploads/${targetFilename}`,
                        size: fileStats.size,
                        mimetype: 'image/jpeg',
                        uploadedAt: new Date().toISOString()
                    });
                    
                    // Save back to media.json
                    fs.writeFileSync(mediaPath, JSON.stringify(mediaData, null, 2), 'utf8');
                    console.log(`Media data updated with article featured image: ${targetFilename}`);
                } catch (mediaError) {
                    console.error('Error adding article image to media.json:', mediaError);
                    // Continue even if there's an error, to not block post creation
                }
            }
        }
        
        const posts = JSON.parse(fs.readFileSync(postsPath));
        posts.unshift(newPost);  // Add to beginning of array (newest first)
        fs.writeFileSync(postsPath, JSON.stringify(posts, null, 2));
        
        req.flash('success_msg', `Artikel "${title}" berhasil dibuat!`);
        // Redirect dengan parameter di URL untuk notifikasi
        return res.redirect(`/admin/posts?success=added&title=${encodeURIComponent(title)}`);
    } catch (error) {
        console.error('Error creating post:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat membuat artikel');
        res.redirect('/admin/posts/new');
    }
});

app.get('/admin/posts/edit/:id', checkAuth, (req, res) => {
    const { id } = req.params;
    
    try {
        const posts = JSON.parse(fs.readFileSync(postsPath));
        const post = posts.find(p => p.id === id);
        
        if (!post) {
            req.flash('error_msg', 'Artikel tidak ditemukan');
            return res.redirect('/admin/posts');
        }
        
        // Cek jika user adalah penulis, pastikan mereka hanya dapat mengedit artikel mereka sendiri
        if (req.session.userRole === 'author' && post.author !== req.session.user.username) {
            req.flash('error_msg', 'Anda tidak memiliki izin untuk mengedit artikel ini');
            return res.redirect('/admin/posts');
        }
        
        const categories = JSON.parse(fs.readFileSync(categoriesPath));
        
        res.render('admin/post-form', { 
            post, 
            categories,
            formTitle: 'Edit Artikel', 
            formAction: `/admin/posts/update/${id}`
        });
    } catch (error) {
        console.error('Error loading post for edit:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat memuat artikel');
        res.redirect('/admin/posts');
    }
});

app.post('/admin/posts/update/:id', checkAuth, upload.single('featuredImage'), async (req, res) => {
    const { id } = req.params;
    const { title, content, category } = req.body;
    
    try {
        if (!title || !content) {
            req.flash('error_msg', 'Judul dan konten wajib diisi');
            return res.redirect(`/admin/posts/edit/${id}`);
        }
        
        const posts = JSON.parse(fs.readFileSync(postsPath));
        const postIndex = posts.findIndex(p => p.id === id);
        
        if (postIndex === -1) {
            req.flash('error_msg', 'Artikel tidak ditemukan');
            return res.redirect('/admin/posts');
        }
        
        // Cek jika user adalah penulis, pastikan mereka hanya dapat mengupdate artikel mereka sendiri
        if (req.session.userRole === 'author' && posts[postIndex].author !== req.session.user.username) {
            req.flash('error_msg', 'Anda tidak memiliki izin untuk mengedit artikel ini');
            return res.redirect('/admin/posts');
        }
        
        // Proses featured image jika diunggah
        if (req.file) {
            const originalPath = path.join(uploadsPath, req.file.filename);
            const targetFilename = req.file.filename + '.jpg';
            const targetPath = path.join(uploadsPath, targetFilename);
            
            const success = await processImage(originalPath, targetPath);
            if (success) {
                posts[postIndex].featuredImage = `/uploads/${targetFilename}`;
                
                // Add to media.json for media manager
                try {
                    let mediaData = [];
                    if (fs.existsSync(mediaPath)) {
                        mediaData = JSON.parse(fs.readFileSync(mediaPath, 'utf8'));
                    }
                    
                    // Add the new file if it doesn't exist already
                    const fileExists = mediaData.some(item => item.filename === targetFilename);
                    if (!fileExists) {
                        const fileStats = fs.statSync(targetPath);
                        mediaData.unshift({
                            id: generateId(),
                            filename: targetFilename,
                            originalname: req.file.originalname || targetFilename,
                            path: `/uploads/${targetFilename}`,
                            size: fileStats.size,
                            mimetype: 'image/jpeg',
                            uploadedAt: new Date().toISOString()
                        });
                        
                        // Save back to media.json
                        fs.writeFileSync(mediaPath, JSON.stringify(mediaData, null, 2), 'utf8');
                        console.log(`Media data updated with article featured image: ${targetFilename}`);
                    }
                } catch (mediaError) {
                    console.error('Error adding article image to media.json:', mediaError);
                    // Continue even if there's an error, to not block post update
                }
            }
        }
        
        // Update post data
        posts[postIndex].title = title;
        posts[postIndex].content = content;
        posts[postIndex].category = category;
        posts[postIndex].lastUpdated = new Date().toISOString();
        
        // Perbarui atau tambahkan slug jika belum ada
        if (!posts[postIndex].slug) {
            posts[postIndex].slug = generateSlug(title);
        }
        
        // Simpan kembali ke file
        fs.writeFileSync(postsPath, JSON.stringify(posts, null, 2), 'utf8');
        
        // Cek jika user ingin tetap di halaman edit
        const stayOnPage = req.body.stayOnPage === 'true';
        
        req.flash('success_msg', 'Artikel berhasil diperbarui!');
        // Redirect dengan parameter untuk notifikasi
        return res.redirect(`/admin/posts?success=edited&title=${encodeURIComponent(title)}`);
    } catch (error) {
        console.error('Error updating post:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat mengupdate artikel');
        res.redirect(`/admin/posts/edit/${id}`);
    }
});

app.get('/admin/posts/delete/:id', checkAuth, (req, res) => {
    const { id } = req.params;
    
    try {
        const posts = JSON.parse(fs.readFileSync(postsPath));
        const postIndex = posts.findIndex(p => p.id === id);
        
        if (postIndex === -1) {
            req.flash('error_msg', 'Artikel tidak ditemukan');
            return res.redirect('/admin/posts');
        }
        
        // Cek jika user adalah penulis, pastikan mereka hanya dapat menghapus artikel mereka sendiri
        if (req.session.userRole === 'author' && posts[postIndex].author !== req.session.user.username) {
            req.flash('error_msg', 'Anda tidak memiliki izin untuk menghapus artikel ini');
            return res.redirect('/admin/posts');
        }
        
        // Simpan judul artikel untuk pesan sukses
        const articleTitle = posts[postIndex].title;
        
        // Hapus gambar featuredImage jika ada
        if (posts[postIndex].featuredImage) {
            const imagePath = path.join(__dirname, 'public', posts[postIndex].featuredImage);
            if (fs.existsSync(imagePath)) {
                fs.unlinkSync(imagePath);
                console.log(`Deleted featured image: ${imagePath}`);
            }
        }
        
        // TAMBAHAN: Ekstrak dan hapus gambar dari konten artikel
        if (posts[postIndex].content) {
            try {
                // Ekstrak semua URL gambar dari konten HTML
                const imgRegex = /<img[^>]+src="([^">]+)"/g;
                const content = posts[postIndex].content;
                let match;
                const contentImages = [];
                
                // Temukan semua URL gambar dalam konten
                while ((match = imgRegex.exec(content)) !== null) {
                    const imgSrc = match[1];
                    // Hanya proses gambar yang disimpan di server (bukan URL eksternal)
                    if (imgSrc.startsWith('/uploads/')) {
                        contentImages.push(imgSrc);
                    }
                }
                
                console.log(`Ditemukan ${contentImages.length} gambar dalam konten artikel`);
                
                // Untuk setiap gambar dalam konten
                for (const imgSrc of contentImages) {
                    // Hapus parameter query jika ada
                    const cleanImgSrc = imgSrc.split('?')[0];
                    const localImgPath = path.join(__dirname, 'public', cleanImgSrc);
                    
                    // Periksa apakah gambar digunakan di artikel lain
                    const otherPosts = posts.filter(p => p.id !== id);
                    const isUsedElsewhere = otherPosts.some(p => {
                        // Cek di featuredImage
                        if (p.featuredImage === cleanImgSrc) return true;
                        
                        // Cek di konten artikel
                        return p.content && p.content.includes(cleanImgSrc);
                    });
                    
                    // Jika gambar tidak digunakan di tempat lain, hapus
                    if (!isUsedElsewhere && fs.existsSync(localImgPath)) {
                        fs.unlinkSync(localImgPath);
                        console.log(`Deleted content image: ${localImgPath}`);
                        
                        // Hapus juga dari media.json
                        if (fs.existsSync(mediaPath)) {
                            try {
                                const filename = path.basename(cleanImgSrc);
                                let mediaData = JSON.parse(fs.readFileSync(mediaPath, 'utf8'));
                                
                                // Filter out file yang dihapus
                                const beforeCount = mediaData.length;
                                mediaData = mediaData.filter(item => item.filename !== filename);
                                
                                // Hitung item yang dihapus
                                const removedCount = beforeCount - mediaData.length;
                                if (removedCount > 0) {
                                    // Update media.json
                                    fs.writeFileSync(mediaPath, JSON.stringify(mediaData, null, 2), 'utf8');
                                    console.log(`${removedCount} item dihapus dari media.json`);
                                }
                            } catch (mediaError) {
                                console.error('Error saat update media.json:', mediaError);
                            }
                        }
                    } else {
                        console.log(`Gambar masih digunakan di artikel lain: ${cleanImgSrc}`);
                    }
                }
            } catch (contentError) {
                console.error('Error saat memproses gambar dalam konten:', contentError);
            }
        }
        
        // Hapus artikel dari array
        posts.splice(postIndex, 1);
        fs.writeFileSync(postsPath, JSON.stringify(posts, null, 2));
        
        req.flash('success_msg', `Artikel "${articleTitle}" berhasil dihapus!`);
        // Redirect dengan parameter di URL untuk menampilkan notifikasi
        return res.redirect(`/admin/posts?success=deleted&title=${encodeURIComponent(articleTitle)}`);
    } catch (error) {
        console.error('Error deleting post:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat menghapus artikel');
        res.redirect('/admin/posts');
    }
});

// Categories management routes
app.get('/admin/categories', checkAuth, checkRole(['admin', 'editor']), (req, res) => {
    let categories = [];
    try {
        categories = JSON.parse(fs.readFileSync(categoriesPath));
        console.log('Categories loaded:', categories);
        
        // Hitung jumlah artikel per kategori
        const posts = JSON.parse(fs.readFileSync(postsPath));
        
        // Tambahkan hitungan artikel ke setiap kategori
        categories = categories.map(category => {
            const articleCount = posts.filter(post => post.category === category.name).length;
            return { ...category, articleCount };
        });
        
    } catch (error) {
        console.error('Error reading categories:', error);
        categories = [];
    }
    
    res.render('admin/categories', { 
        categories,
        success_msg: req.flash('success_msg'),
        error_msg: req.flash('error_msg')
    });
});

// Media management routes
app.get('/admin/media', checkAuth, (req, res) => {
    try {
        // Membaca data dari media.json
        let media = [];
        if (fs.existsSync(mediaPath)) {
            const mediaData = fs.readFileSync(mediaPath, 'utf8');
            if (mediaData) {
                try {
                    media = JSON.parse(mediaData);
                } catch (error) {
                    console.error('Error parsing media.json:', error);
                }
            }
        }
        
        // Render halaman media dengan data
        res.render('admin/media', {
            user: req.session.user,
            userRole: req.session.userRole,
            media: media,
            success_msg: req.flash('success_msg'),
            error_msg: req.flash('error_msg')
        });
    } catch (error) {
        console.error('Error rendering media page:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat memuat halaman media');
        res.redirect('/admin/dashboard');
    }
});

// Media upload endpoint
app.post('/admin/media/upload', checkAuth, checkRole(['admin', 'editor', 'author']), upload.array('mediaFile', 10), async (req, res) => {
    try {
        console.log('Media upload request received');
        
        if (!req.files || req.files.length === 0) {
            console.log('No files uploaded');
            return res.status(400).json({ success: false, message: 'Tidak ada file yang diunggah' });
        }
        
        console.log(`Processing ${req.files.length} uploaded files`);
        const processedFiles = [];
        
        // Proses setiap file yang diupload
        for (const file of req.files) {
            console.log(`Processing file: ${file.originalname}`);
            const originalPath = path.join(uploadsPath, file.filename);
            const targetFilename = file.filename + '.jpg';
            const targetPath = path.join(uploadsPath, targetFilename);
            
            // Proses dan resize gambar
            const success = await processImage(originalPath, targetPath);
            
            if (success) {
                const fileStats = fs.statSync(targetPath);
                console.log(`File processed successfully: ${targetFilename}, size: ${fileStats.size} bytes`);
                
                processedFiles.push({
                    id: generateId(),
                    filename: targetFilename,
                    originalname: file.originalname,
                    path: `/uploads/${targetFilename}`,
                    size: fileStats.size,
                    mimetype: 'image/jpeg',
                    uploadedAt: new Date().toISOString()
                });
            } else {
                console.error(`Failed to process file: ${file.originalname}`);
            }
        }
        
        if (processedFiles.length === 0) {
            return res.status(500).json({ 
                success: false, 
                message: 'Gagal memproses file yang diunggah. Coba lagi nanti.' 
            });
        }
        
        // Baca file media.json yang ada
        let mediaData = [];
        try {
            if (fs.existsSync(mediaPath)) {
                mediaData = JSON.parse(fs.readFileSync(mediaPath, 'utf8'));
            }
        } catch (error) {
            console.error('Error reading media.json:', error);
            // Lanjutkan dengan array kosong jika file tidak dapat dibaca
        }
        
        // Tambahkan file yang baru diupload
        mediaData = [...processedFiles, ...mediaData];
        
        // Simpan kembali ke media.json
        fs.writeFileSync(mediaPath, JSON.stringify(mediaData, null, 2), 'utf8');
        console.log(`Media data updated with ${processedFiles.length} new files`);
        
        // Sync media.json dengan folder uploads
        syncMediaJson();
        
        return res.json({ 
            success: true, 
            message: `${processedFiles.length} file berhasil diunggah`, 
            files: processedFiles 
        });
    } catch (error) {
        console.error('Error in media upload:', error);
        return res.status(500).json({ 
            success: false, 
            message: 'Terjadi kesalahan saat mengunggah file: ' + (error.message || 'Unknown error') 
        });
    }
});

// Media delete endpoint
app.delete('/admin/media/delete/:filename', checkAuth, checkRole(['admin', 'editor']), (req, res) => {
    try {
        const { filename } = req.params;
        const filePath = path.join(uploadsPath, filename);
        
        console.log(`Mencoba menghapus media (API): ${filename}`);
        
        if (fs.existsSync(filePath)) {
            // Hapus file dari sistem
            fs.unlinkSync(filePath);
            console.log(`File media berhasil dihapus: ${filePath}`);
            
            // Hapus juga dari media.json
            if (fs.existsSync(mediaPath)) {
                const mediaData = fs.readFileSync(mediaPath, 'utf8');
                if (mediaData) {
                    let mediaFiles = JSON.parse(mediaData);
                    const beforeCount = mediaFiles.length;
                    mediaFiles = mediaFiles.filter(file => file.filename !== filename);
                    const removedCount = beforeCount - mediaFiles.length;
                    console.log(`${removedCount} item dihapus dari media.json (API)`);
                    
                    // Save back to media.json
                    fs.writeFileSync(mediaPath, JSON.stringify(mediaFiles, null, 2), 'utf8');
                }
            }
            
            // Juga, periksa apakah file ini digunakan di artikel mana pun
            if (fs.existsSync(postsPath)) {
                try {
                    const postsData = fs.readFileSync(postsPath, 'utf8');
                    let posts = JSON.parse(postsData);
                    
                    // Cek apakah ada artikel yang menggunakan gambar ini
                    const relatedPosts = posts.filter(post => 
                        post.featuredImage === `/uploads/${filename}` || 
                        post.featuredImage === `/uploads/${filename}?size=1600x900`
                    );
                    
                    // Log info tentang artikel terkait
                    if (relatedPosts.length > 0) {
                        console.log(`Gambar digunakan di ${relatedPosts.length} artikel. Menghapus referensi...`);
                        
                        // Hapus referensi gambar dari artikel
                        posts = posts.map(post => {
                            if (post.featuredImage === `/uploads/${filename}` || 
                                post.featuredImage === `/uploads/${filename}?size=1600x900`) {
                                post.featuredImage = '';
                                console.log(`Referensi gambar dihapus dari artikel ID: ${post.id}`);
                            }
                            return post;
                        });
                        
                        // Simpan kembali data posts
                        fs.writeFileSync(postsPath, JSON.stringify(posts, null, 2), 'utf8');
                    } else {
                        console.log(`Gambar tidak digunakan di artikel manapun.`);
                    }
                } catch (postsError) {
                    console.error('Error saat memeriksa artikel terkait:', postsError);
                }
            }
            
            return res.json({ success: true, message: 'Media berhasil dihapus' });
        } else {
            return res.status(404).json({ success: false, message: 'File tidak ditemukan' });
        }
    } catch (error) {
        console.error('Error deleting media:', error);
        return res.status(500).json({ success: false, message: 'Terjadi kesalahan saat menghapus media' });
    }
});

// Endpoint untuk menghapus media via GET
app.get('/admin/media/delete/:filename', checkAuth, (req, res) => {
    try {
        const { filename } = req.params;
        console.log(`Mencoba menghapus media (GET): ${filename}`);
        
        // Hapus file fisik
        const filePath = path.join(uploadsPath, filename);
        if (fs.existsSync(filePath)) {
            fs.unlinkSync(filePath);
            console.log(`File media berhasil dihapus: ${filePath}`);
        } else {
            console.log(`File media tidak ditemukan: ${filePath}`);
        }
        
        // Hapus referensi dari media.json
        if (fs.existsSync(mediaPath)) {
            let mediaData = JSON.parse(fs.readFileSync(mediaPath, 'utf8'));
            const beforeCount = mediaData.length;
            
            // Filter out file yang dihapus
            mediaData = mediaData.filter(item => item.filename !== filename);
            
            // Hitung item yang dihapus
            const removedCount = beforeCount - mediaData.length;
            
            // Update media.json
            fs.writeFileSync(mediaPath, JSON.stringify(mediaData, null, 2), 'utf8');
            console.log(`${removedCount} item dihapus dari media.json (GET)`);
        }
        
        // Juga, periksa apakah file ini digunakan di artikel mana pun
        if (fs.existsSync(postsPath)) {
            try {
                const postsData = fs.readFileSync(postsPath, 'utf8');
                let posts = JSON.parse(postsData);
                
                // Cek apakah ada artikel yang menggunakan gambar ini
                const relatedPosts = posts.filter(post => 
                    post.featuredImage === `/uploads/${filename}` || 
                    post.featuredImage === `/uploads/${filename}?size=1600x900`
                );
                
                // Log info tentang artikel terkait
                if (relatedPosts.length > 0) {
                    console.log(`Gambar digunakan di ${relatedPosts.length} artikel. Menghapus referensi...`);
                    
                    // Hapus referensi gambar dari artikel
                    posts = posts.map(post => {
                        if (post.featuredImage === `/uploads/${filename}` || 
                            post.featuredImage === `/uploads/${filename}?size=1600x900`) {
                            post.featuredImage = '';
                            console.log(`Referensi gambar dihapus dari artikel ID: ${post.id}`);
                        }
                        return post;
                    });
                    
                    // Simpan kembali data posts
                    fs.writeFileSync(postsPath, JSON.stringify(posts, null, 2), 'utf8');
                } else {
                    console.log(`Gambar tidak digunakan di artikel manapun.`);
                }
            } catch (postsError) {
                console.error('Error saat memeriksa artikel terkait:', postsError);
            }
        }
        
        req.flash('success_msg', 'Media berhasil dihapus');
    } catch (error) {
        console.error('Error deleting media:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat menghapus media');
    }
    
    // Redirect back to media page
    res.redirect('/admin/media');
});

app.post('/admin/categories/create', checkAuth, checkRole(['admin', 'editor']), (req, res) => {
    const { name, slug } = req.body;
    
    if (!name || name.trim() === '') {
        req.flash('error_msg', 'Nama game tidak boleh kosong');
        return res.redirect('/admin/categories');
    }
    
    let categories = [];
    try {
        categories = JSON.parse(fs.readFileSync(categoriesPath));
    } catch (error) {
        console.error('Error reading categories:', error);
        categories = [];
    }
    
    // Cek apakah game dengan nama yang sama sudah ada
    if (categories.some(cat => cat.name.toLowerCase() === name.toLowerCase())) {
        req.flash('error_msg', 'Game dengan nama tersebut sudah ada');
        return res.redirect('/admin/categories');
    }
    
    // Generate slug jika tidak disediakan
    let categorySlug = slug;
    if (!categorySlug || categorySlug.trim() === '') {
        categorySlug = name.toLowerCase()
            .replace(/\s+/g, '-')           // Replace spaces with -
            .replace(/[^\w\-]+/g, '')       // Remove all non-word chars
            .replace(/\-\-+/g, '-')         // Replace multiple - with single -
            .replace(/^-+/, '')             // Trim - from start of text
            .replace(/-+$/, '');            // Trim - from end of text
    }
    
    // Cek apakah slug sudah digunakan
    if (categories.some(cat => cat.slug === categorySlug)) {
        categorySlug = `${categorySlug}-${Date.now().toString().slice(-4)}`;
    }
    
    const newCategory = {
        id: Date.now().toString(),
        name,
        slug: categorySlug,
        createdAt: new Date().toISOString()
    };
    
    categories.push(newCategory);
    fs.writeFileSync(categoriesPath, JSON.stringify(categories));
    
    req.flash('success_msg', 'Game berhasil ditambahkan');
    res.redirect('/admin/categories');
});

app.post('/admin/categories/update/:id', checkAuth, checkRole(['admin', 'editor']), (req, res) => {
    const { id } = req.params;
    const { name, slug } = req.body;
    
    if (!name || name.trim() === '') {
        req.flash('error_msg', 'Nama game tidak boleh kosong');
        return res.redirect('/admin/categories');
    }
    
    let categories = [];
    try {
        categories = JSON.parse(fs.readFileSync(categoriesPath));
    } catch (error) {
        console.error('Error reading categories:', error);
        categories = [];
    }
    
    // Cek apakah game dengan nama yang sama sudah ada (kecuali game yang sedang diedit)
    if (categories.some(cat => cat.name.toLowerCase() === name.toLowerCase() && cat.id !== id)) {
        req.flash('error_msg', 'Game dengan nama tersebut sudah ada');
        return res.redirect('/admin/categories');
    }
    
    // Generate slug jika tidak disediakan
    let categorySlug = slug;
    if (!categorySlug || categorySlug.trim() === '') {
        categorySlug = name.toLowerCase()
            .replace(/\s+/g, '-')
            .replace(/[^\w\-]+/g, '')
            .replace(/\-\-+/g, '-')
            .replace(/^-+/, '')
            .replace(/-+$/, '');
    }
    
    // Cek apakah slug sudah digunakan (kecuali oleh game yang sedang diedit)
    if (categories.some(cat => cat.slug === categorySlug && cat.id !== id)) {
        categorySlug = `${categorySlug}-${Date.now().toString().slice(-4)}`;
    }
    
    const categoryIndex = categories.findIndex(cat => cat.id === id);
    if (categoryIndex === -1) {
        req.flash('error_msg', 'Game tidak ditemukan');
        return res.redirect('/admin/categories');
    }
    
    categories[categoryIndex] = {
        ...categories[categoryIndex],
        name,
        slug: categorySlug,
        updatedAt: new Date().toISOString()
    };
    
    fs.writeFileSync(categoriesPath, JSON.stringify(categories));
    
    req.flash('success_msg', 'Game berhasil diperbarui');
    res.redirect('/admin/categories');
});

app.get('/admin/categories/delete/:id', checkAuth, checkRole(['admin']), (req, res) => {
    const { id } = req.params;
    
    let categories = [];
    try {
        categories = JSON.parse(fs.readFileSync(categoriesPath));
    } catch (error) {
        console.error('Error reading categories:', error);
        categories = [];
    }
    
    const filteredCategories = categories.filter(cat => cat.id !== id);
    
    if (filteredCategories.length === categories.length) {
        req.flash('error_msg', 'Game tidak ditemukan');
        return res.redirect('/admin/categories');
    }
    
    fs.writeFileSync(categoriesPath, JSON.stringify(filteredCategories));
    
    // Update posts yang menggunakan game ini menjadi "Umum"
    let posts = [];
    try {
        posts = JSON.parse(fs.readFileSync(postsPath));
        const category = categories.find(cat => cat.id === id);
        
        if (category) {
            posts = posts.map(post => {
                if (post.category === category.name) {
                    return { ...post, category: 'Umum' };
                }
                return post;
            });
            
            fs.writeFileSync(postsPath, JSON.stringify(posts));
        }
    } catch (error) {
        console.error('Error updating posts categories:', error);
    }
    
    req.flash('success_msg', 'Game berhasil dihapus');
    res.redirect('/admin/categories');
});

// Comments management routes
app.get('/admin/comments', checkAuth, (req, res) => {
    let comments = [];
    let posts = [];
    
    try {
        comments = JSON.parse(fs.readFileSync(commentsPath));
        posts = JSON.parse(fs.readFileSync(postsPath));
        
        // Sort comments by date (newest first)
        comments.sort((a, b) => {
            const dateA = new Date(a.date || a.createdAt);
            const dateB = new Date(b.date || b.createdAt);
            return dateB - dateA;
        });
        
        // Filter comments for authors - only show comments on their own articles
        if (req.session.userRole === 'author') {
            // Get post IDs for posts authored by this user
            const authorPostIds = posts
                .filter(post => post.author === req.session.user.username)
                .map(post => post.id);
            
            // Filter comments to only include those on the author's posts
            comments = comments.filter(comment => authorPostIds.includes(comment.postId));
        }
        
    } catch (error) {
        console.error('Error reading comments or posts:', error);
        comments = [];
        posts = [];
    }
    
    res.render('admin/comments', { 
        comments,
        posts,
        userPostsOnly: req.session.userRole === 'author',
        success_msg: req.flash('success_msg'),
        error_msg: req.flash('error_msg')
    });
});

// Approve comment
app.get('/admin/comments/approve/:id', checkAuth, (req, res) => {
    const { id } = req.params;
    let comments = [];
    let posts = [];
    
    try {
        comments = JSON.parse(fs.readFileSync(commentsPath));
        posts = JSON.parse(fs.readFileSync(postsPath));
        
        // Find the comment
        const commentIndex = comments.findIndex(comment => comment.id === id);
        
        if (commentIndex !== -1) {
            const comment = comments[commentIndex];
            
            // For authors, check if the comment belongs to their article
            if (req.session.userRole === 'author') {
                // Get post IDs for posts authored by this user
                const authorPostIds = posts
                    .filter(post => post.author === req.session.user.username)
                    .map(post => post.id);
                
                // Check if the comment's postId is in the author's posts
                if (!authorPostIds.includes(comment.postId)) {
                    req.flash('error_msg', 'Anda tidak memiliki izin untuk menyetujui komentar ini');
                    return res.redirect('/admin/comments');
                }
            }
            
            comments[commentIndex].status = 'approved';
            fs.writeFileSync(commentsPath, JSON.stringify(comments, null, 2));
            req.flash('success_msg', 'Komentar berhasil disetujui');
        } else {
            req.flash('error_msg', 'Komentar tidak ditemukan');
        }
        
    } catch (error) {
        console.error('Error approving comment:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat menyetujui komentar');
    }
    
    res.redirect('/admin/comments');
});

// Mark comment as spam
app.get('/admin/comments/mark-spam/:id', checkAuth, (req, res) => {
    const { id } = req.params;
    let comments = [];
    
    try {
        // Check if user is author - if so, deny access
        if (req.session.userRole === 'author') {
            req.flash('error_msg', 'Anda tidak memiliki izin untuk menandai komentar sebagai spam');
            return res.redirect('/admin/comments');
        }
        
        comments = JSON.parse(fs.readFileSync(commentsPath));
        
        // Find and update the comment status
        const commentIndex = comments.findIndex(comment => comment.id === id);
        
        if (commentIndex !== -1) {
            comments[commentIndex].status = 'spam';
            fs.writeFileSync(commentsPath, JSON.stringify(comments, null, 2));
            req.flash('success_msg', 'Komentar berhasil ditandai sebagai spam');
        } else {
            req.flash('error_msg', 'Komentar tidak ditemukan');
        }
        
    } catch (error) {
        console.error('Error marking comment as spam:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat menandai komentar sebagai spam');
    }
    
    res.redirect('/admin/comments');
});

// Delete comment
app.get('/admin/comments/delete/:id', checkAuth, (req, res) => {
    const { id } = req.params;
    let comments = [];
    
    try {
        // Check if user is author - if so, deny access
        if (req.session.userRole === 'author') {
            req.flash('error_msg', 'Anda tidak memiliki izin untuk menghapus komentar');
            return res.redirect('/admin/comments');
        }
        
        comments = JSON.parse(fs.readFileSync(commentsPath));
        
        // Filter out the comment to delete
        const newComments = comments.filter(comment => comment.id !== id);
        
        if (newComments.length < comments.length) {
            fs.writeFileSync(commentsPath, JSON.stringify(newComments, null, 2));
            req.flash('success_msg', 'Komentar berhasil dihapus');
        } else {
            req.flash('error_msg', 'Komentar tidak ditemukan');
        }
        
    } catch (error) {
        console.error('Error deleting comment:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat menghapus komentar');
    }
    
    res.redirect('/admin/comments');
});

// Admin Users Route
app.get('/admin/users', checkAuth, checkRole(['admin']), (req, res) => {
    try {
        // Baca file users.json
        let users = [];
        if (fs.existsSync(usersPath)) {
            const usersData = fs.readFileSync(usersPath, 'utf8');
            users = JSON.parse(usersData);
        }
        
        // Urutkan pengguna (admin dulu, kemudian sortir berdasarkan username)
        users.sort((a, b) => {
            // Admin selalu di atas
            if (a.role === 'admin' && b.role !== 'admin') return -1;
            if (a.role !== 'admin' && b.role === 'admin') return 1;
            
            // Urutkan berdasarkan username jika role sama
            return a.username.localeCompare(b.username);
        });
        
        res.render('admin/users', { 
            users,
            currentUsername: req.session.user.username,
            success_msg: req.flash('success_msg'),
            error_msg: req.flash('error_msg')
        });
    } catch (error) {
        console.error('Error loading users:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat memuat data pengguna');
        res.render('admin/users', { 
            users: [],
            currentUsername: req.session.user.username,
            success_msg: req.flash('success_msg'),
            error_msg: req.flash('error_msg') 
        });
    }
});

// Add User Route
app.post('/admin/users/add', checkAuth, checkRole(['admin']), (req, res) => {
    console.log('Receiving user add request with body:', req.body);
    
    try {
        // Pastikan req.body tidak undefined atau null
        if (!req.body) {
            console.error('Empty request body received');
            req.flash('error_msg', 'Data tidak diterima. Silakan coba lagi.');
            return res.redirect('/admin/users');
        }
        
        const { username, email, displayName, role, password, confirmPassword } = req.body;
        
        console.log('Parsed data from request:', {
            hasUsername: !!username,
            hasEmail: !!email,
            hasDisplayName: !!displayName,
            hasRole: !!role,
            hasPassword: !!password,
            hasConfirmPassword: !!confirmPassword
        });
        
        // Validasi input
        if (!username || !password) {
            console.log('Validation failed: Missing username or password');
            req.flash('error_msg', 'Username dan password wajib diisi');
            return res.redirect('/admin/users');
        }
        
        if (password !== confirmPassword) {
            console.log('Validation failed: Password mismatch');
            req.flash('error_msg', 'Password dan konfirmasi password tidak cocok');
            return res.redirect('/admin/users');
        }
        
        if (password.length < 6) {
            console.log('Validation failed: Password too short');
            req.flash('error_msg', 'Password minimal 6 karakter');
            return res.redirect('/admin/users');
        }
        
        // Baca file users.json
        let users = [];
        if (fs.existsSync(usersPath)) {
            try {
                const usersData = fs.readFileSync(usersPath, 'utf8');
                if (usersData && usersData.trim() !== '') {
                    users = JSON.parse(usersData);
                    console.log(`Loaded ${users.length} existing users`);
                } else {
                    console.log('users.json is empty, initializing empty array');
                    users = [];
                }
            } catch (parseError) {
                console.error('Error parsing users.json:', parseError);
                // Initialize empty array if parse fails
                users = [];
            }
        } else {
            console.log('users.json does not exist, creating new file');
            // Ensure data directory exists
            if (!fs.existsSync(path.dirname(usersPath))) {
                fs.mkdirSync(path.dirname(usersPath), { recursive: true });
            }
            fs.writeFileSync(usersPath, JSON.stringify([]));
        }
        
        // Cek username sudah digunakan atau belum
        if (users.some(user => user.username === username)) {
            console.log(`Username ${username} already exists`);
            req.flash('error_msg', 'Username sudah digunakan');
            return res.redirect('/admin/users');
        }
        
        // Cek email sudah digunakan atau belum (hanya jika email diisi)
        if (email && users.some(user => user.email === email)) {
            console.log(`Email ${email} already exists`);
            req.flash('error_msg', 'Email sudah digunakan');
            return res.redirect('/admin/users');
        }
        
        // Hash password
        const hashedPassword = bcrypt.hashSync(password, 10);
        
        // Buat user baru
        const newUser = {
            username,
            email: email || '',
            displayName: displayName || username,
            password: hashedPassword,
            role: role || 'author',
            status: 'active',
            createdAt: new Date().toISOString()
        };
        
        console.log('Created new user object:', {
            username: newUser.username,
            displayName: newUser.displayName,
            role: newUser.role,
            status: newUser.status
        });
        
        // Tambahkan user baru ke array
        users.push(newUser);
        
        // Simpan ke file
        try {
            fs.writeFileSync(usersPath, JSON.stringify(users, null, 2), 'utf8');
            console.log('Successfully saved updated users.json file');
            
            req.flash('success_msg', 'Pengguna berhasil ditambahkan');
            return res.redirect(`/admin/users?action=added&username=${encodeURIComponent(username)}`);
        } catch (writeError) {
            console.error('Error writing to users.json:', writeError);
            req.flash('error_msg', `Terjadi kesalahan saat menyimpan data: ${writeError.message}`);
            return res.redirect('/admin/users');
        }
    } catch (error) {
        console.error('Unexpected error adding user:', error);
        req.flash('error_msg', 'Terjadi kesalahan tidak terduga: ' + error.message);
        return res.redirect('/admin/users');
    }
});

// Activate User Route
app.get('/admin/users/activate/:username', checkAuth, checkRole(['admin']), (req, res) => {
    const { username } = req.params;
    
    try {
        // Baca file users.json
        let users = [];
        if (fs.existsSync(usersPath)) {
            const usersData = fs.readFileSync(usersPath, 'utf8');
            users = JSON.parse(usersData);
        }
        
        // Cari user berdasarkan username
        const userIndex = users.findIndex(user => user.username === username);
        
        if (userIndex === -1) {
            req.flash('error_msg', 'Pengguna tidak ditemukan');
            return res.redirect('/admin/users');
        }
        
        // Update status user
        users[userIndex].status = 'active';
        
        // Simpan ke file
        fs.writeFileSync(usersPath, JSON.stringify(users, null, 2), 'utf8');
        
        req.flash('success_msg', 'Pengguna berhasil diaktifkan');
        res.redirect(`/admin/users?action=activated&username=${encodeURIComponent(username)}`);
    } catch (error) {
        console.error('Error activating user:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat mengaktifkan pengguna');
        res.redirect('/admin/users');
    }
});

// Deactivate User Route
app.get('/admin/users/deactivate/:username', checkAuth, checkRole(['admin']), (req, res) => {
    const { username } = req.params;
    
    try {
        // Baca file users.json
        let users = [];
        if (fs.existsSync(usersPath)) {
            const usersData = fs.readFileSync(usersPath, 'utf8');
            users = JSON.parse(usersData);
        }
        
        // Cari user berdasarkan username
        const userIndex = users.findIndex(user => user.username === username);
        
        if (userIndex === -1) {
            req.flash('error_msg', 'Pengguna tidak ditemukan');
            return res.redirect('/admin/users');
        }
        
        // Tidak bisa menonaktifkan diri sendiri
        if (username === req.session.user.username) {
            req.flash('error_msg', 'Anda tidak dapat menonaktifkan akun Anda sendiri');
            return res.redirect('/admin/users');
        }
        
        // Update status user
        users[userIndex].status = 'inactive';
        
        // Simpan ke file
        fs.writeFileSync(usersPath, JSON.stringify(users, null, 2), 'utf8');
        
        req.flash('success_msg', 'Pengguna berhasil dinonaktifkan');
        res.redirect(`/admin/users?action=deactivated&username=${encodeURIComponent(username)}`);
    } catch (error) {
        console.error('Error deactivating user:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat menonaktifkan pengguna');
        res.redirect('/admin/users');
    }
});

// Delete User Route
app.get('/admin/users/delete/:username', checkAuth, checkRole(['admin']), (req, res) => {
    const { username } = req.params;
    
    try {
        // Baca file users.json
        let users = [];
        if (fs.existsSync(usersPath)) {
            const usersData = fs.readFileSync(usersPath, 'utf8');
            users = JSON.parse(usersData);
        }
        
        // Tidak bisa menghapus diri sendiri
        if (username === req.session.user.username) {
            req.flash('error_msg', 'Anda tidak dapat menghapus akun Anda sendiri');
            return res.redirect('/admin/users');
        }
        
        // Filter users untuk menghapus user tertentu
        const newUsers = users.filter(user => user.username !== username);
        
        // Cek apakah ada user yang dihapus
        if (newUsers.length === users.length) {
            req.flash('error_msg', 'Pengguna tidak ditemukan');
            return res.redirect('/admin/users');
        }
        
        // Simpan ke file
        fs.writeFileSync(usersPath, JSON.stringify(newUsers, null, 2), 'utf8');
        
        req.flash('success_msg', 'Pengguna berhasil dihapus');
        res.redirect(`/admin/users?action=deleted&username=${encodeURIComponent(username)}`);
    } catch (error) {
        console.error('Error deleting user:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat menghapus pengguna');
        res.redirect('/admin/users');
    }
});

// Unlock User Route (Reset failed login attempts)
app.post('/admin/users/unlock', checkAuth, checkRole(['admin']), (req, res) => {
    const { username } = req.body;
    
    try {
        // Baca file users.json
        let users = [];
        if (fs.existsSync(usersPath)) {
            const usersData = fs.readFileSync(usersPath, 'utf8');
            users = JSON.parse(usersData);
        }
        
        // Cari user berdasarkan username
        const userIndex = users.findIndex(user => user.username === username);
        
        if (userIndex === -1) {
            req.flash('error_msg', 'Pengguna tidak ditemukan');
            return res.redirect('/admin/users');
        }
        
        // Update status user
        if (users[userIndex].status === 'locked') {
            users[userIndex].status = 'active';
            users[userIndex].loginAttempts = 0;
            
            // Simpan ke file
            fs.writeFileSync(usersPath, JSON.stringify(users, null, 2), 'utf8');
            
            // Reset failed login attempts
            if (failedLoginAttempts[username]) {
                delete failedLoginAttempts[username];
            }
            
            req.flash('success_msg', 'Kunci akun pengguna berhasil dibuka');
        } else {
            req.flash('error_msg', 'Akun pengguna tidak dalam keadaan terkunci');
        }
        
        res.redirect(`/admin/users?action=unlocked&username=${encodeURIComponent(username)}`);
    } catch (error) {
        console.error('Error unlocking user:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat membuka kunci akun pengguna');
        res.redirect('/admin/users');
    }
});

// Edit User Route
app.post('/admin/users/edit', checkAuth, checkRole(['admin']), (req, res) => {
    const { username, email, displayName, role, password, confirmPassword } = req.body;
    
    try {
        // Validasi input
        if (!username || !email) {
            req.flash('error_msg', 'Username dan email wajib diisi');
            return res.redirect('/admin/users');
        }
        
        // Validasi password jika diisi
        if (password && password !== confirmPassword) {
            req.flash('error_msg', 'Password dan konfirmasi password tidak cocok');
            return res.redirect('/admin/users');
        }
        
        // Baca file users.json
        let users = [];
        if (fs.existsSync(usersPath)) {
            const usersData = fs.readFileSync(usersPath, 'utf8');
            users = JSON.parse(usersData);
        }
        
        // Cari user berdasarkan username
        const userIndex = users.findIndex(user => user.username === username);
        
        if (userIndex === -1) {
            req.flash('error_msg', 'Pengguna tidak ditemukan');
            return res.redirect('/admin/users');
        }
        
        // Update data user
        users[userIndex].email = email;
        users[userIndex].displayName = displayName || username;
        
        // Hanya update role jika bukan diri sendiri
        if (username !== req.session.user.username) {
            users[userIndex].role = role || 'author';
        }
        
        // Update password jika diisi
        if (password) {
            const hashedPassword = bcrypt.hashSync(password, 10);
            users[userIndex].password = hashedPassword;
            users[userIndex].lastPasswordUpdate = new Date().toISOString();
        }
        
        // Simpan ke file
        fs.writeFileSync(usersPath, JSON.stringify(users, null, 2), 'utf8');
        
        req.flash('success_msg', 'Data pengguna berhasil diperbarui');
        res.redirect(`/admin/users?action=edited&username=${encodeURIComponent(username)}`);
    } catch (error) {
        console.error('Error editing user:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat memperbarui data pengguna');
        res.redirect('/admin/users');
    }
});

// Reset Password Route
app.post('/admin/users/reset-password', checkAuth, checkRole(['admin']), (req, res) => {
    const { username, password, confirmPassword } = req.body;
    
    try {
        // Validasi input
        if (!username || !password) {
            req.flash('error_msg', 'Username dan password baru wajib diisi');
            return res.redirect('/admin/users');
        }
        
        if (password !== confirmPassword) {
            req.flash('error_msg', 'Password baru dan konfirmasi password tidak cocok');
            return res.redirect('/admin/users');
        }
        
        // Baca file users.json
        let users = [];
        if (fs.existsSync(usersPath)) {
            const usersData = fs.readFileSync(usersPath, 'utf8');
            users = JSON.parse(usersData);
        }
        
        // Cari user berdasarkan username
        const userIndex = users.findIndex(user => user.username === username);
        
        if (userIndex === -1) {
            req.flash('error_msg', 'Pengguna tidak ditemukan');
            return res.redirect('/admin/users');
        }
        
        // Hash password baru
        const hashedPassword = bcrypt.hashSync(password, 10);
        
        // Update password user
        users[userIndex].password = hashedPassword;
        users[userIndex].lastPasswordUpdate = new Date().toISOString();
        
        // Simpan ke file
        fs.writeFileSync(usersPath, JSON.stringify(users, null, 2), 'utf8');
        
        req.flash('success_msg', 'Password pengguna berhasil direset');
        res.redirect(`/admin/users?action=password-reset&username=${encodeURIComponent(username)}`);
    } catch (error) {
        console.error('Error resetting user password:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat mereset password pengguna');
        res.redirect('/admin/users');
    }
});

// Admin Settings Route
app.get('/admin/settings', checkAuth, checkRole(['admin']), (req, res) => {
    try {
        // Load settings from file
        let settings = {};
        if (fs.existsSync(settingsPath)) {
            settings = JSON.parse(fs.readFileSync(settingsPath, 'utf8'));
        }
        
        // Load categories for reference
        let categories = [];
        if (fs.existsSync(categoriesPath)) {
            categories = JSON.parse(fs.readFileSync(categoriesPath, 'utf8'));
        }
        
        // Load IP whitelist
        let ipWhitelist = [];
        const dataDir = path.join(__dirname, 'data');
        const ipWhitelistPath = path.join(dataDir, 'ip-whitelist.json');
        
        if (fs.existsSync(ipWhitelistPath)) {
            ipWhitelist = JSON.parse(fs.readFileSync(ipWhitelistPath, 'utf8'));
        } else {
            // Default whitelist
            ipWhitelist = ['127.0.0.1', '::1'];
            
            // Pastikan folder data ada sebelum menulis file default
            if (!fs.existsSync(dataDir)) {
                fs.mkdirSync(dataDir, { recursive: true });
            }
            
            // Tulis file default jika belum ada
            fs.writeFileSync(ipWhitelistPath, JSON.stringify(ipWhitelist, null, 2));
        }
        
        // Mendapatkan IP pengguna saat ini untuk ditampilkan di view
        const clientIp = req.ip || 
                       req.connection.remoteAddress || 
                       req.socket.remoteAddress ||
                       (req.connection.socket ? req.connection.socket.remoteAddress : null);
        
        // Menghapus IPv6 prefix jika ada
        const currentIp = (clientIp || '').replace(/^::ffff:/, '');
        
        res.render('admin/settings', {
            settings,
            categories,
            ipWhitelist,
            currentIp,
            user: req.session.user,
            userRole: req.session.userRole,
            success_msg: req.flash('success_msg'),
            error_msg: req.flash('error_msg')
        });
    } catch (error) {
        console.error('Error loading settings:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat memuat pengaturan.');
        res.redirect('/admin/dashboard');
    }
});

// Update settings
app.post('/admin/settings/update', checkAuth, checkRole(['admin']), (req, res) => {
    try {
        const { section } = req.body;
        let settings = {};
        
        // Load current settings
        if (fs.existsSync(settingsPath)) {
            settings = JSON.parse(fs.readFileSync(settingsPath, 'utf8'));
        }
        
        // Update specified section
        switch (section) {
            case 'general':
                settings.general = {
                    siteName: req.body.siteName || 'MahaSlot',
                    siteDescription: req.body.siteDescription || '',
                    siteUrl: req.body.siteUrl || '',
                    adminEmail: req.body.adminEmail || '',
                    timezone: req.body.timezone || 'Asia/Jakarta'
                };
                break;
            case 'content':
                settings.content = {
                    postsPerPage: parseInt(req.body.postsPerPage) || 10,
                    defaultCategory: req.body.defaultCategory || '',
                    enableComments: req.body.enableComments === 'on',
                    moderateComments: req.body.moderateComments === 'on'
                };
                break;
            case 'design':
                settings.design = {
                    logoUrl: req.body.logoUrl || '/img/logo.png',
                    faviconUrl: req.body.faviconUrl || '/img/favicon.ico',
                    marqueeText: req.body.marqueeText || '🔥 DIBAYAR LUNAS 500 JUTA! 🔥★💰 MAXWIN 200 JUTA SLOT GACOR! 💰★🎁 BONUS NEW MEMBER 100% 🎁★🏆 MAHASLOT SITUS TERPERCAYA INDONESIA 🏆★👑 RATUSAN MEMBER MENANG BESAR SETIAP HARI! 👑',
                    artikelTitle: req.body.artikelTitle || 'MAHASLOT: SITUS TERPERCAYA PEMBAYARAN JACKPOT BESAR',
                    artikelContent: req.body.artikelContent || 'MAHASLOT telah membuktikan dirinya sebagai situs judi online paling terpercaya di Indonesia dengan rekam jejak pembayaran jackpot besar yang konsisten dan cepat. Sebagai platform gaming premium, MAHASLOT menawarkan berbagai macam permainan slot online dari provider ternama seperti Pragmatic Play, Habanero, Microgaming, dan PG Soft dengan RTP tinggi dan potensi kemenangan maksimal.\n\nKomitmen kami terhadap pelayanan pelanggan terbaik dan transparansi pembayaran telah menjadikan MAHASLOT sebagai pilihan utama para pemain slot di Indonesia. Dengan layanan customer service 24/7, proses deposit dan withdraw yang cepat, serta jaminan keamanan data pribadi, kami memastikan pengalaman bermain yang nyaman dan aman bagi semua member.',
                    // Tombol CTA Settings
                    ctaButton1Text: req.body.ctaButton1Text || 'DAFTAR SEKARANG',
                    ctaButton1Url: req.body.ctaButton1Url || '#',
                    ctaButton1Icon: req.body.ctaButton1Icon || 'fas fa-user-plus',
                    ctaButton2Text: req.body.ctaButton2Text || 'HUBUNGI KAMI',
                    ctaButton2Url: req.body.ctaButton2Url || '#',
                    ctaButton2Icon: req.body.ctaButton2Icon || 'fas fa-comments'
                };
                break;
            case 'seo':
                settings.seo = {
                    metaTitle: req.body.metaTitle || settings.seo.metaTitle || '',
                    metaDescription: req.body.metaDescription || settings.seo.metaDescription || '',
                    metaKeywords: req.body.metaKeywords || settings.seo.metaKeywords || '',
                    metaRobots: req.body.metaRobots || 'index,follow',
                    enableSitemap: false, // Tetap dinonaktifkan
                    enableRobots: false // Tetap dinonaktifkan
                };
                break;
            case 'social':
                settings.social = {
                    facebookUrl: req.body.facebookUrl || '',
                    twitterUrl: req.body.twitterUrl || '',
                    instagramUrl: req.body.instagramUrl || '',
                    youtubeUrl: req.body.youtubeUrl || '',
                    enableSocialSharing: req.body.enableSocialSharing === 'on'
                };
                break;
            default:
                return res.status(400).json({ success: false, message: 'Invalid section' });
        }
        
        // Save settings to file
        fs.writeFileSync(settingsPath, JSON.stringify(settings, null, 4), 'utf8');
        
        // Send JSON response for AJAX request
        if (req.xhr) {
            return res.json({ success: true, message: 'Settings updated successfully' });
        }
        
        req.flash('success_msg', 'Pengaturan berhasil diperbarui');
        res.redirect('/admin/settings');
    } catch (error) {
        console.error('Error updating settings:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat memperbarui pengaturan.');
        res.redirect('/admin/settings');
    }
});

// Update IP whitelist settings
app.post('/admin/settings/update-whitelist', checkAuth, checkRole(['admin']), (req, res) => {
    try {
        const { ips } = req.body;
        
        // Validasi data
        if (!ips || !Array.isArray(ips) || ips.length === 0) {
            return res.status(400).json({ success: false, message: 'Data IP tidak valid' });
        }
        
        // Pastikan minimal ada localhost dalam daftar
        let hasLocalhost = false;
        for (const ip of ips) {
            if (ip === '127.0.0.1' || ip === '::1') {
                hasLocalhost = true;
                break;
            }
        }
        
        if (!hasLocalhost) {
            // Tambahkan localhost jika tidak ada
            ips.push('127.0.0.1');
            ips.push('::1');
        }
        
        // Pastikan folder data ada
        const dataDir = path.join(__dirname, 'data');
        if (!fs.existsSync(dataDir)) {
            fs.mkdirSync(dataDir, { recursive: true });
        }
        
        // Simpan ke file
        const ipWhitelistPath = path.join(dataDir, 'ip-whitelist.json');
        fs.writeFileSync(ipWhitelistPath, JSON.stringify(ips, null, 2));
        
        // Log perubahan
        console.log('IP Whitelist diperbarui oleh', req.session.user.username);
        
        return res.json({ success: true, message: 'Whitelist IP berhasil diperbarui' });
    } catch (error) {
        console.error('Error updating IP whitelist:', error);
        return res.status(500).json({ success: false, message: 'Terjadi kesalahan saat memperbarui whitelist IP' });
    }
});

// Upload Logo
app.post('/admin/settings/upload-logo', checkAuth, checkRole(['admin']), upload.single('logoImage'), async (req, res) => {
    try {
        if (!req.file) {
            return res.status(400).json({ success: false, message: 'Tidak ada file yang diunggah' });
        }
        
        const originalPath = path.join(uploadsPath, req.file.filename);
        const targetFilename = 'logo_' + Date.now() + '.png';
        const targetPath = path.join(uploadsPath, targetFilename);
        
        // Process image with larger dimensions for logo (preserve aspect ratio)
        const success = await processImage(originalPath, targetPath, {
            width: 400,       // Increased width for larger logo
            height: 200,      // Proportional height
            format: 'png',    // PNG format for better quality
            quality: 100,     // Max quality
            fit: sharp.fit.inside  // Preserve aspect ratio
        });
        
        if (success) {
            // Update logo URL in settings
            let settings = {};
            if (fs.existsSync(settingsPath)) {
                settings = JSON.parse(fs.readFileSync(settingsPath, 'utf8'));
            }
            
            // Ensure design section exists
            if (!settings.design) {
                settings.design = {};
            }
            
            const fileUrl = `/uploads/${targetFilename}`;
            settings.design.logoUrl = fileUrl;
            
            // Save settings
            fs.writeFileSync(settingsPath, JSON.stringify(settings, null, 4), 'utf8');
            
            // Return success response
            return res.json({ 
                success: true, 
                message: 'Logo berhasil diunggah', 
                fileUrl: fileUrl 
            });
        } else {
            return res.status(500).json({ success: false, message: 'Gagal memproses gambar' });
        }
    } catch (error) {
        console.error('Error uploading logo:', error);
        return res.status(500).json({ success: false, message: error.message });
    }
});

// Upload Favicon
app.post('/admin/settings/upload-favicon', checkAuth, checkRole(['admin']), upload.single('faviconImage'), async (req, res) => {
    try {
        if (!req.file) {
            return res.status(400).json({ success: false, message: 'Tidak ada file yang diunggah' });
        }
        
        const originalPath = path.join(uploadsPath, req.file.filename);
        const targetFilename = 'favicon_' + Date.now() + '.png';
        const targetPath = path.join(uploadsPath, targetFilename);
        
        // Process image (resize for favicon - standardize to 32x32 pixels)
        const success = await processImage(originalPath, targetPath, { 
            width: 32, 
            height: 32, 
            format: 'png',
            quality: 100,
            fit: sharp.fit.cover // Ensure exact dimensions
        });
        
        if (success) {
            // Update favicon URL in settings
            let settings = {};
            if (fs.existsSync(settingsPath)) {
                settings = JSON.parse(fs.readFileSync(settingsPath, 'utf8'));
            }
            
            // Ensure design section exists
            if (!settings.design) {
                settings.design = {};
            }
            
            const fileUrl = `/uploads/${targetFilename}`;
            settings.design.faviconUrl = fileUrl;
            
            // Save settings
            fs.writeFileSync(settingsPath, JSON.stringify(settings, null, 4), 'utf8');
            
            // Return success response
            return res.json({ 
                success: true, 
                message: 'Favicon berhasil diunggah', 
                fileUrl: fileUrl 
            });
        } else {
            return res.status(500).json({ success: false, message: 'Gagal memproses gambar' });
        }
    } catch (error) {
        console.error('Error uploading favicon:', error);
        return res.status(500).json({ success: false, message: error.message });
    }
});

// Frontend routes
app.get('/', (req, res) => {
    try {
        // Ambil pengaturan
        const settingsData = fs.readFileSync(settingsPath, 'utf8');
        const settings = JSON.parse(settingsData);
        
        // Ambil kategori
        let categories = [];
        if (fs.existsSync(categoriesPath)) {
            const categoriesData = fs.readFileSync(categoriesPath, 'utf8');
            if (categoriesData) {
                categories = JSON.parse(categoriesData);
            }
        }
        
        // Ambil semua artikel
        let posts = [];
        if (fs.existsSync(postsPath)) {
            const postsData = fs.readFileSync(postsPath, 'utf8');
            if (postsData) {
                posts = JSON.parse(postsData);
                
                // Urutkan berdasarkan tanggal terbaru
                posts.sort((a, b) => {
                    const dateA = new Date(a.date);
                    const dateB = new Date(b.date);
                    return dateB - dateA;
                });
            }
        }
        
        // Pagination
        const page = parseInt(req.query.page) || 1;
        const postsPerPage = settings.content && settings.content.postsPerPage ? settings.content.postsPerPage : 8;
        const totalPosts = posts.length;
        const totalPages = Math.ceil(totalPosts / postsPerPage);
        
        const startIndex = (page - 1) * postsPerPage;
        const endIndex = startIndex + postsPerPage;
        const paginatedPosts = posts.slice(startIndex, endIndex);
        
        res.render('home', { 
            settings, 
            categories, 
            posts: paginatedPosts,
            currentPage: page,
            totalPages,
            totalPosts
        });
    } catch (error) {
        console.error('Error loading home page:', error);
        res.status(500).send('Terjadi kesalahan saat memuat halaman');
    }
});

app.get('/category/:slug', (req, res) => {
    try {
        const { slug } = req.params;
        
        // Ambil pengaturan
        const settingsData = fs.readFileSync(settingsPath, 'utf8');
        const settings = JSON.parse(settingsData);
        
        // Ambil kategori
        let categories = [];
        let category = null;
        
        if (fs.existsSync(categoriesPath)) {
            const categoriesData = fs.readFileSync(categoriesPath, 'utf8');
            if (categoriesData) {
                categories = JSON.parse(categoriesData);
                category = categories.find(cat => cat.slug === slug || cat.id === slug);
            }
        }
        
        if (!category) {
            return res.status(404).send('Kategori tidak ditemukan');
        }
        
        // Ambil semua artikel
        let posts = [];
        if (fs.existsSync(postsPath)) {
            const postsData = fs.readFileSync(postsPath, 'utf8');
            if (postsData) {
                const allPosts = JSON.parse(postsData);
                
                // Filter berdasarkan kategori (periksa baik dengan nama kategori atau slug)
                posts = allPosts.filter(post => {
                    if (typeof post.category === 'object') {
                        return post.category.id === category.id;
                    } else {
                        // Backward compatibility: bisa menggunakan name atau slug
                        return post.category === category.name || 
                               post.category === category.slug || 
                               post.category === category.id;
                    }
                });
                
                // Urutkan berdasarkan tanggal terbaru
                posts.sort((a, b) => {
                    const dateA = new Date(a.date);
                    const dateB = new Date(b.date);
                    return dateB - dateA;
                });
            }
        }
        
        // Pagination
        const page = parseInt(req.query.page) || 1;
        const postsPerPage = settings.content.postsPerPage || 9;
        const totalPosts = posts.length;
        const totalPages = Math.ceil(totalPosts / postsPerPage);
        
        const startIndex = (page - 1) * postsPerPage;
        const endIndex = startIndex + postsPerPage;
        const paginatedPosts = posts.slice(startIndex, endIndex);
        
        res.render('category', { 
            settings, 
            categories, 
            category, 
            posts: paginatedPosts,
            currentPage: page,
            totalPages,
            totalPosts
        });
    } catch (error) {
        console.error('Error loading category page:', error);
        res.status(500).send('Terjadi kesalahan saat memuat halaman kategori');
    }
});

app.get('/post/:id', (req, res) => {
    try {
        const { id } = req.params;
        
        // Ambil pengaturan
        const settingsData = fs.readFileSync(settingsPath, 'utf8');
        const settings = JSON.parse(settingsData);
        
        // Ambil kategori
        let categories = [];
        if (fs.existsSync(categoriesPath)) {
            const categoriesData = fs.readFileSync(categoriesPath, 'utf8');
            if (categoriesData) {
                categories = JSON.parse(categoriesData);
            }
        }
        
        // Ambil artikel
        let posts = [];
        let post = null;
        
        if (fs.existsSync(postsPath)) {
            const postsData = fs.readFileSync(postsPath, 'utf8');
            if (postsData) {
                posts = JSON.parse(postsData);
                // Cari post baik berdasarkan ID atau slug (backward compatibility)
                post = posts.find(p => p.id === id || p.slug === id);
                
                // Jika artikel ditemukan, tambahkan views
                if (post) {
                    post.views = (post.views || 0) + 1;
                    
                    // Simpan kembali ke file
                    fs.writeFileSync(postsPath, JSON.stringify(posts, null, 2), 'utf8');
                }
            }
        }
        
        if (!post) {
            return res.status(404).send('Artikel tidak ditemukan');
        }
        
        // Cari artikel sebelumnya dan selanjutnya berdasarkan tanggal posting
        // 1. Urutkan semua artikel berdasarkan tanggal (terbaru ke terlama)
        const sortedPosts = [...posts].sort((a, b) => {
            const dateA = new Date(a.date);
            const dateB = new Date(b.date);
            return dateB - dateA; // Urutan dari terbaru (DESC)
        });
        
        console.log('Total posts in DB:', posts.length);
        console.log('Current post ID:', post.id);
        
        // 2. Temukan indeks post saat ini dalam array yang sudah diurutkan
        const currentIndex = sortedPosts.findIndex(p => p.id === post.id);
        console.log('Current post index in sorted array:', currentIndex);
        
        // 3. Tentukan post sebelumnya (yang lebih lama) dan selanjutnya (yang lebih baru)
        let previousPost = null;
        let nextPost = null;
        
        if (currentIndex !== -1) {
            // Post sebelumnya adalah post dengan indeks + 1 (post yang lebih lama)
            if (currentIndex < sortedPosts.length - 1) {
                previousPost = sortedPosts[currentIndex + 1];
                console.log('Previous post found:', previousPost.id);
            }
            
            // Post selanjutnya adalah post dengan indeks - 1 (post yang lebih baru)
            if (currentIndex > 0) {
                nextPost = sortedPosts[currentIndex - 1];
                console.log('Next post found:', nextPost.id);
            }
        }
        
        // Jika masih tidak ditemukan post sebelumnya/selanjutnya, gunakan dummy data
        if (!previousPost && posts.length > 1) {
            // Cari post lain yang bukan post saat ini untuk dijadikan post sebelumnya
            const otherPosts = posts.filter(p => p.id !== post.id);
            if (otherPosts.length > 0) {
                previousPost = otherPosts[0]; // Ambil post pertama yang ditemukan
                console.log('Using fallback for previous post:', previousPost.id);
            }
        }
        
        if (!nextPost && posts.length > 1) {
            // Cari post lain yang bukan post saat ini dan juga bukan post sebelumnya untuk dijadikan post selanjutnya
            const otherPosts = posts.filter(p => p.id !== post.id && (!previousPost || p.id !== previousPost.id));
            if (otherPosts.length > 0) {
                nextPost = otherPosts[0]; // Ambil post pertama yang ditemukan
                console.log('Using fallback for next post:', nextPost.id);
            }
        }
        
        // Ambil artikel terkait (dari kategori yang sama)
        let relatedPosts = [];
        if (post.category) {
            // Tentukan kategori ID atau nilai untuk perbandingan
            let categoryIdentifier;
            
            if (typeof post.category === 'object') {
                categoryIdentifier = post.category.id;
            } else {
                categoryIdentifier = post.category;
            }
            
            relatedPosts = posts.filter(p => {
                // Skip post saat ini
                if (p.id === post.id) return false;
                
                // Periksa apakah kategori cocok
                if (typeof p.category === 'object') {
                    return p.category.id === categoryIdentifier || 
                           p.category.slug === categoryIdentifier || 
                           p.category.name === categoryIdentifier;
                } else {
                    return p.category === categoryIdentifier;
                }
            })
            .sort((a, b) => {
                const dateA = new Date(a.date);
                const dateB = new Date(b.date);
                return dateB - dateA;
            })
            .slice(0, 3);  // Ambil 3 artikel terkait
        }
        
        // Ambil komentar
        let comments = [];
        if (fs.existsSync(commentsPath)) {
            const commentsData = fs.readFileSync(commentsPath, 'utf8');
            if (commentsData) {
                const allComments = JSON.parse(commentsData);
                
                // Filter komentar untuk artikel ini dan yang sudah disetujui
                comments = allComments.filter(comment => 
                    comment.postId === post.id && comment.status === 'approved'
                );
                
                // Urutkan berdasarkan tanggal
                comments.sort((a, b) => {
                    const dateA = new Date(a.date);
                    const dateB = new Date(b.date);
                    return dateB - dateA;
                });
            }
        }
        
        console.log('Rendering with real previous post:', previousPost ? previousPost.id : 'none');
        console.log('Rendering with real next post:', nextPost ? nextPost.id : 'none');
        
        res.render('post', { 
            settings, 
            categories, 
            post, 
            posts,
            relatedPosts,
            previousPost, // Gunakan artikel asli yang ditemukan
            nextPost,     // Gunakan artikel asli yang ditemukan
            comments,
            success_msg: req.flash('success_msg'),
            error_msg: req.flash('error_msg')
        });
    } catch (error) {
        console.error('Error loading post page:', error);
        res.status(500).send('Terjadi kesalahan saat memuat halaman artikel');
    }
});

app.get('/search', (req, res) => {
    try {
        const { q } = req.query;
        
        // Ambil pengaturan
        const settingsData = fs.readFileSync(settingsPath, 'utf8');
        const settings = JSON.parse(settingsData);
        
        // Ambil kategori
        let categories = [];
        if (fs.existsSync(categoriesPath)) {
            const categoriesData = fs.readFileSync(categoriesPath, 'utf8');
            if (categoriesData) {
                categories = JSON.parse(categoriesData);
            }
        }
        
        // Ambil semua artikel
        let posts = [];
        if (fs.existsSync(postsPath)) {
            const postsData = fs.readFileSync(postsPath, 'utf8');
            if (postsData) {
                const allPosts = JSON.parse(postsData);
                
                // Jika ada query pencarian
                if (q) {
                    const searchTerms = q.toLowerCase().split(' ');
                    
                    // Filter posts berdasarkan judul artikel dan kategori
                    posts = allPosts.filter(post => {
                        const title = post.title.toLowerCase();
                        
                        // Mendapatkan nama kategori dari post
                        let categoryName = '';
                        if (typeof post.category === 'object' && post.category) {
                            categoryName = post.category.name || '';
                        } else if (typeof post.category === 'string') {
                            categoryName = post.category;
                            
                            // Cari nama kategori jika hanya memiliki slug/id
                            if (categories && categories.length > 0) {
                                const foundCategory = categories.find(cat => 
                                    cat.slug === post.category || cat.id === post.category
                                );
                                if (foundCategory) {
                                    categoryName = foundCategory.name;
                                }
                            }
                        }
                        
                        categoryName = categoryName.toLowerCase();
                        
                        // Cek jika setiap istilah pencarian ada di judul atau nama kategori
                        return searchTerms.some(term => 
                            title.includes(term) || categoryName.includes(term)
                        );
                    });
                    
                    // Sort berdasarkan relevansi - posts yang judulnya atau kategorinya mengandung lebih banyak kata pencarian di awal
                    posts.sort((a, b) => {
                        const titleA = a.title.toLowerCase();
                        const titleB = b.title.toLowerCase();
                        
                        // Mendapatkan nama kategori dari post A
                        let categoryNameA = '';
                        if (typeof a.category === 'object' && a.category) {
                            categoryNameA = a.category.name || '';
                        } else if (typeof a.category === 'string') {
                            categoryNameA = a.category;
                            
                            // Cari nama kategori jika hanya memiliki slug/id
                            if (categories && categories.length > 0) {
                                const foundCategory = categories.find(cat => 
                                    cat.slug === a.category || cat.id === a.category
                                );
                                if (foundCategory) {
                                    categoryNameA = foundCategory.name;
                                }
                            }
                        }
                        categoryNameA = categoryNameA.toLowerCase();
                        
                        // Mendapatkan nama kategori dari post B
                        let categoryNameB = '';
                        if (typeof b.category === 'object' && b.category) {
                            categoryNameB = b.category.name || '';
                        } else if (typeof b.category === 'string') {
                            categoryNameB = b.category;
                            
                            // Cari nama kategori jika hanya memiliki slug/id
                            if (categories && categories.length > 0) {
                                const foundCategory = categories.find(cat => 
                                    cat.slug === b.category || cat.id === b.category
                                );
                                if (foundCategory) {
                                    categoryNameB = foundCategory.name;
                                }
                            }
                        }
                        categoryNameB = categoryNameB.toLowerCase();
                        
                        // Hitung berapa banyak kata pencarian yang cocok dengan judul dan kategori
                        const matchCountA = searchTerms.filter(term => 
                            titleA.includes(term) || categoryNameA.includes(term)
                        ).length;
                        const matchCountB = searchTerms.filter(term => 
                            titleB.includes(term) || categoryNameB.includes(term)
                        ).length;
                        
                        // Prioritaskan jumlah kecocokan
                        if (matchCountA !== matchCountB) {
                            return matchCountB - matchCountA;
                        }
                        
                        // Jika kecocokan sama, prioritaskan yang judulnya lebih pendek
                        // (kemungkinan besar lebih relevan)
                        return titleA.length - titleB.length;
                    });
                }
                
                // Urutkan berdasarkan tanggal terbaru
                posts.sort((a, b) => {
                    const dateA = new Date(a.date);
                    const dateB = new Date(b.date);
                    return dateB - dateA;
                });
            }
        }
        
        // Pagination
        const page = parseInt(req.query.page) || 1;
        const postsPerPage = settings.content.postsPerPage || 9;
        const totalPosts = posts.length;
        const totalPages = Math.ceil(totalPosts / postsPerPage);
        
        const startIndex = (page - 1) * postsPerPage;
        const endIndex = startIndex + postsPerPage;
        const paginatedPosts = posts.slice(startIndex, endIndex);
        
        res.render('search', { 
            settings, 
            categories, 
            posts: paginatedPosts,
            searchQuery: q,
            currentPage: page,
            totalPages,
            totalPosts
        });
    } catch (error) {
        console.error('Error loading search page:', error);
        res.status(500).send('Terjadi kesalahan saat memuat halaman pencarian');
    }
});

app.get('/about', (req, res) => {
    try {
        // Ambil pengaturan
        const settingsData = fs.readFileSync(settingsPath, 'utf8');
        const settings = JSON.parse(settingsData);
        
        // Ambil kategori
        let categories = [];
        if (fs.existsSync(categoriesPath)) {
            const categoriesData = fs.readFileSync(categoriesPath, 'utf8');
            if (categoriesData) {
                categories = JSON.parse(categoriesData);
            }
        }
        
        res.render('about', { settings, categories });
    } catch (error) {
        console.error('Error loading about page:', error);
        res.status(500).send('Terjadi kesalahan saat memuat halaman tentang kami');
    }
});

app.get('/contact', (req, res) => {
    try {
        // Ambil pengaturan
        const settingsData = fs.readFileSync(settingsPath, 'utf8');
        const settings = JSON.parse(settingsData);
        
        // Ambil kategori
        let categories = [];
        if (fs.existsSync(categoriesPath)) {
            const categoriesData = fs.readFileSync(categoriesPath, 'utf8');
            if (categoriesData) {
                categories = JSON.parse(categoriesData);
            }
        }
        
        res.render('contact', { 
            settings, 
            categories,
            success_msg: req.flash('success_msg'),
            error_msg: req.flash('error_msg')
        });
    } catch (error) {
        console.error('Error loading contact page:', error);
        res.status(500).send('Terjadi kesalahan saat memuat halaman kontak');
    }
});

app.post('/contact/submit', (req, res) => {
    try {
        const { name, email, subject, message } = req.body;
        
        // Validasi input
        if (!name || !email || !message) {
            req.flash('error_msg', 'Semua field wajib diisi');
            return res.redirect('/contact');
        }
        
        // Simpan pesan kontak ke file
        const contactMessage = {
            id: generateId(),
            name,
            email,
            subject: subject || 'Tidak ada subjek',
            message,
            date: new Date().toISOString(),
            status: 'unread'
        };
        
        let messages = [];
        if (fs.existsSync(contactsPath)) {
            const messagesData = fs.readFileSync(contactsPath, 'utf8');
            if (messagesData) {
                messages = JSON.parse(messagesData);
            }
        }
        
        // Tambahkan pesan baru
        messages.push(contactMessage);
        
        // Simpan ke file
        fs.writeFileSync(contactsPath, JSON.stringify(messages, null, 2), 'utf8');
        
        req.flash('success_msg', 'Pesan Anda telah terkirim! Kami akan menghubungi Anda segera.');
        res.redirect('/contact');
    } catch (error) {
        console.error('Error submitting contact form:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat mengirim pesan');
        res.redirect('/contact');
    }
});

// Add public route to submit a comment
app.post('/post/:id/comment', (req, res) => {
    const { id } = req.params;
    const { author, email, content } = req.body;
    
    // Basic validation
    if (!author || !content) {
        req.flash('error_msg', 'Nama dan komentar harus diisi');
        return res.redirect(`/post/${id}`);
    }
    
    try {
        // Load posts to check if the post exists
        const posts = JSON.parse(fs.readFileSync(postsPath));
        const post = posts.find(p => p.id === id);
        
        if (!post) {
            req.flash('error_msg', 'Artikel tidak ditemukan');
            return res.redirect('/');
        }
        
        // Load existing comments
        let comments = [];
        try {
            if (fs.existsSync(commentsPath)) {
                const commentsData = fs.readFileSync(commentsPath, 'utf8');
                if (commentsData) {
                    comments = JSON.parse(commentsData);
                }
            }
        } catch (err) {
            console.error('Error reading comments:', err);
            comments = [];
        }
        
        // Create new comment
        const newComment = {
            id: generateId(),
            postId: id,
            author,
            email: email || '',
            content,
            date: new Date().toISOString(),
            status: settings.content.moderateComments ? 'pending' : 'approved'
        };
        
        // Add to comments array
        comments.push(newComment);
        
        // Save the updated comments
        fs.writeFileSync(commentsPath, JSON.stringify(comments));
        
        if (settings.content.moderateComments) {
            req.flash('success_msg', 'Komentar Anda telah dikirimkan dan sedang menunggu persetujuan');
        } else {
            req.flash('success_msg', 'Komentar Anda telah dipublikasikan');
        }
        res.redirect(`/post/${id}`);
        
    } catch (error) {
        console.error('Error adding comment:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat menambahkan komentar');
        res.redirect(`/post/${id}`);
    }
});

// Admin route default
app.get('/admin', (req, res) => {
    if (req.session.isAuthenticated) {
        return res.redirect('/admin/dashboard');
    }
    return res.redirect('/admin/login');
});

// Fungsi untuk memastikan media.json sync dengan folder public/uploads
function syncMediaJson() {
    try {
        // Jika folder uploads tidak ada, buat folder uploads
        if (!fs.existsSync(uploadsPath)) {
            fs.mkdirSync(uploadsPath, { recursive: true });
        }
        
        // Baca file dari folder uploads - hanya file JPG (hasil resize)
        const files = fs.readdirSync(uploadsPath).filter(file => file.endsWith('.jpg'));
        const uploadsFiles = files.map(filename => {
            const filePath = path.join(uploadsPath, filename);
            const stats = fs.statSync(filePath);
            
            return {
                filename: filename,
                path: `/uploads/${filename}`,
                size: stats.size,
                mimetype: 'image/jpeg',
                mtime: stats.mtime.toISOString()
            };
        });
        
        // Baca data dari media.json jika ada
        let mediaFiles = [];
        if (fs.existsSync(mediaPath)) {
            const mediaData = fs.readFileSync(mediaPath, 'utf8');
            if (mediaData) {
                try {
                    mediaFiles = JSON.parse(mediaData);
                } catch (error) {
                    console.error('Error parsing media.json:', error);
                    mediaFiles = [];
                }
            }
        }
        
        // Tambahkan file yang ada di uploads tapi tidak ada di media.json
        const mediaFilenames = mediaFiles.map(file => file.filename);
        for (const file of uploadsFiles) {
            if (!mediaFilenames.includes(file.filename)) {
                mediaFiles.push({
                    id: generateId(),
                    filename: file.filename,
                    originalname: file.filename,
                    path: file.path,
                    size: file.size,
                    mimetype: file.mimetype,
                    uploadedAt: file.mtime
                });
            }
        }
        
        // Hapus file yang ada di media.json tapi tidak ada di uploads
        mediaFiles = mediaFiles.filter(file => {
            return uploadsFiles.some(uploadFile => uploadFile.filename === file.filename);
        });
        
        // Simpan kembali ke media.json
        fs.writeFileSync(mediaPath, JSON.stringify(mediaFiles, null, 2), 'utf8');
        console.log('Media.json berhasil disinkronkan dengan folder uploads');
        
        // Tambahan: Sinkronkan gambar dari artikel yang sudah ada
        try {
            if (fs.existsSync(postsPath)) {
                const postsData = fs.readFileSync(postsPath, 'utf8');
                if (postsData) {
                    const posts = JSON.parse(postsData);
                    
                    // Cari semua gambar artikel yang belum ada di media.json
                    for (const post of posts) {
                        if (post.featuredImage && post.featuredImage.startsWith('/uploads/')) {
                            const filename = post.featuredImage.replace('/uploads/', '');
                            
                            // Periksa apakah gambar sudah ada di media.json
                            const fileExists = mediaFiles.some(item => item.filename === filename);
                            
                            if (!fileExists && fs.existsSync(path.join(uploadsPath, filename))) {
                                const fileStats = fs.statSync(path.join(uploadsPath, filename));
                                
                                mediaFiles.push({
                                    id: generateId(),
                                    filename: filename,
                                    originalname: filename,
                                    path: `/uploads/${filename}`,
                                    size: fileStats.size,
                                    mimetype: 'image/jpeg',
                                    uploadedAt: fileStats.mtime.toISOString()
                                });
                                
                                console.log(`Added existing article image to media.json: ${filename}`);
                            }
                        }
                    }
                    
                    // Simpan kembali ke media.json jika ada perubahan
                    fs.writeFileSync(mediaPath, JSON.stringify(mediaFiles, null, 2), 'utf8');
                }
            }
        } catch (error) {
            console.error('Error syncing article images with media.json:', error);
        }
        
    } catch (error) {
        console.error('Error syncing media.json:', error);
    }
}

// Jalankan sinkronisasi saat aplikasi pertama dijalankan
syncMediaJson();

// Route untuk melayani gambar dengan ukuran kustom
app.get('/uploads/:filename', (req, res) => {
    const { filename } = req.params;
    const { size } = req.query;
    
    const filePath = path.join(uploadsPath, filename);
    
    // Jika file tidak ditemukan, kembalikan 404
    if (!fs.existsSync(filePath)) {
        return res.status(404).send('File tidak ditemukan');
    }
    
    // Jika tidak ada parameter size, langsung kirim file asli
    if (!size) {
        return res.sendFile(filePath);
    }
    
    // Jika ada parameter size=1600x900, lakukan resize gambar
    if (size === '1600x900') {
        const dimensions = size.split('x');
        const width = parseInt(dimensions[0]);
        const height = parseInt(dimensions[1]);
        
        // Tentukan jenis konten berdasarkan ekstensi file
        const ext = path.extname(filename).toLowerCase();
        let contentType = 'image/jpeg'; // default
        
        if (ext === '.png') contentType = 'image/png';
        else if (ext === '.gif') contentType = 'image/gif';
        else if (ext === '.webp') contentType = 'image/webp';
        
        res.setHeader('Content-Type', contentType);
        
        // Proses gambar dengan sharp
        sharp(filePath)
            .resize(width, height, {
                fit: sharp.fit.cover, // Gunakan cover untuk mempertahankan aspek rasio dan memenuhi ukuran
                position: 'center'
            })
            .toBuffer()
            .then(data => {
                res.send(data);
            })
            .catch(err => {
                console.error('Error resizing image:', err);
                res.sendFile(filePath); // Fallback ke file asli jika gagal resize
            });
    } else {
        // Jika parameter size tidak dikenali, kirim file asli
        res.sendFile(filePath);
    }
});

// Migrasi slug untuk artikel lama yang belum memiliki slug
function migratePostSlugs() {
    try {
        if (fs.existsSync(postsPath)) {
            const postsData = fs.readFileSync(postsPath, 'utf8');
            if (postsData) {
                let posts = JSON.parse(postsData);
                let needsUpdate = false;
                
                posts.forEach(post => {
                    if (!post.slug && post.title) {
                        post.slug = generateSlug(post.title);
                        needsUpdate = true;
                    }
                });
                
                if (needsUpdate) {
                    fs.writeFileSync(postsPath, JSON.stringify(posts, null, 2), 'utf8');
                    console.log('Migrasi slug untuk artikel lama berhasil dilakukan');
                }
            }
        }
    } catch (error) {
        console.error('Error during post slug migration:', error);
    }
}

// Jalankan migrasi saat aplikasi start
migratePostSlugs();

// Middleware untuk menangani 404 (Not Found)
app.use((req, res, next) => {
    // Baca settings jika ada
    let settings = {};
    try {
        const settingsData = fs.readFileSync(settingsPath, 'utf8');
        settings = JSON.parse(settingsData);
    } catch (error) {
        console.error('Error loading settings for 404 page:', error);
        // Gunakan settings default jika tidak bisa membaca file
        settings = {
            general: { siteName: 'Website' },
            design: { 
                logoUrl: '/uploads/logo.png',
                faviconUrl: '/favicon.ico'
            },
            social: {}
        };
    }
    
    // Render halaman 404 dengan settings
    res.status(404).render('404', {
        message: 'Halaman yang Anda cari tidak ditemukan',
        settings: settings
    });
});

// Middleware untuk menangani error (500)
app.use((err, req, res, next) => {
    console.error('Server Error:', err);
    
    // Baca settings jika ada
    let settings = {};
    try {
        const settingsData = fs.readFileSync(settingsPath, 'utf8');
        settings = JSON.parse(settingsData);
    } catch (error) {
        console.error('Error loading settings for error page:', error);
        // Gunakan settings default jika tidak bisa membaca file
        settings = {
            general: { siteName: 'Website' },
            design: { 
                logoUrl: '/uploads/logo.png',
                faviconUrl: '/favicon.ico'
            },
            social: {}
        };
    }
    
    // Render halaman error dengan settings
    res.status(500).render('500', {
        message: 'Terjadi kesalahan pada server',
        settings: settings
    });
});

// Media cleanup routes
app.get('/admin/media/cleanup', checkAuth, checkRole(['admin']), (req, res) => {
    console.log('ACCESS: /admin/media/cleanup route accessed');
    res.render('admin/media-cleanup', {
        user: req.session.user,
        userRole: req.session.userRole,
        success_msg: req.flash('success_msg'),
        error_msg: req.flash('error_msg')
    });
});

app.post('/admin/media/cleanup/run', checkAuth, checkRole(['admin']), async (req, res) => {
    try {
        // Jalankan pembersihan media
        const result = await cleanUnusedMedia();
        
        if (result.success) {
            req.flash('success_msg', `Pembersihan media berhasil! ${result.stats.deletedFiles} file dihapus dari ${result.stats.unusedFiles} file yang tidak digunakan. Waktu proses: ${result.executionTime.toFixed(2)} detik.`);
        } else {
            req.flash('error_msg', `Terjadi kesalahan: ${result.message}`);
        }
        
        return res.redirect('/admin/media/cleanup');
    } catch (error) {
        console.error('Error saat pembersihan media:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat membersihkan media');
        return res.redirect('/admin/media/cleanup');
    }
});

// JSON API untuk pembersihan media
app.post('/admin/api/media/cleanup', checkAuth, checkRole(['admin']), async (req, res) => {
    try {
        // Jalankan pembersihan media
        const result = await cleanUnusedMedia();
        return res.json(result);
    } catch (error) {
        console.error('Error saat pembersihan media:', error);
        return res.status(500).json({
            success: false,
            message: error.message || 'Terjadi kesalahan saat membersihkan media',
            error: error.toString()
        });
    }
});

// Catch-all 404 route untuk debugging
app.use((req, res, next) => {
    console.log(`404 ERROR: Route not found: ${req.originalUrl}`);
    res.status(404).send(`Route not found: ${req.originalUrl}`);
});

// Server start
const port = process.env.PORT || 3004;

app.listen(port, () => {
    console.log(`Server berjalan di port ${port}`);
    console.log(`Akses melalui: http://localhost:${port}`);
});

/**
 * Fungsi untuk membersihkan file media yang tidak digunakan
 * @return {Object} Hasil pembersihan dengan statistik
 */
async function cleanUnusedMedia() {
    try {
        console.log('Memulai pembersihan media...');
        const startTime = Date.now();
        
        // Statistik untuk dilaporkan
        const stats = {
            totalFiles: 0,
            usedFiles: 0,
            unusedFiles: 0,
            deletedFiles: 0,
            errors: 0,
            mediaJsonUpdated: 0
        };
        
        // 1. Baca semua file dari direktori uploads
        if (!fs.existsSync(uploadsPath)) {
            return { success: false, message: 'Direktori uploads tidak ditemukan' };
        }
        
        const allFiles = fs.readdirSync(uploadsPath)
            .filter(file => {
                // Filter hanya file gambar
                const ext = path.extname(file).toLowerCase();
                return ['.jpg', '.jpeg', '.png', '.gif', '.webp', '.svg'].includes(ext);
            })
            .map(file => ({
                filename: file,
                path: `/uploads/${file}`,
                localPath: path.join(uploadsPath, file),
                inUse: false // akan diupdate nanti
            }));
            
        stats.totalFiles = allFiles.length;
        console.log(`Total file gambar ditemukan: ${stats.totalFiles}`);
        
        // 2. Baca semua artikel untuk mendapatkan gambar yang digunakan
        let allPosts = [];
        if (fs.existsSync(postsPath)) {
            try {
                allPosts = JSON.parse(fs.readFileSync(postsPath, 'utf8'));
            } catch (err) {
                console.error('Error membaca file posts.json:', err);
                return { success: false, message: 'Error membaca file posts.json' };
            }
        }
        
        // Set untuk menyimpan semua path gambar yang digunakan
        const usedImagePaths = new Set();
        
        // 3. Periksa featuredImage di setiap artikel
        allPosts.forEach(post => {
            if (post.featuredImage) {
                // Hapus query string
                const cleanPath = post.featuredImage.split('?')[0];
                usedImagePaths.add(cleanPath);
            }
        });
        
        // 4. Periksa gambar dalam konten artikel
        allPosts.forEach(post => {
            if (post.content) {
                // Ekstrak semua URL gambar dari konten HTML
                const imgRegex = /<img[^>]+src="([^">]+)"/g;
                let match;
                
                while ((match = imgRegex.exec(post.content)) !== null) {
                    const imgSrc = match[1];
                    // Hanya proses gambar yang disimpan di server (bukan URL eksternal)
                    if (imgSrc.startsWith('/uploads/')) {
                        // Hapus query string jika ada
                        const cleanPath = imgSrc.split('?')[0];
                        usedImagePaths.add(cleanPath);
                    }
                }
            }
        });
        
        // 5. Periksa gambar yang digunakan dalam pengaturan
        if (fs.existsSync(settingsPath)) {
            try {
                const settings = JSON.parse(fs.readFileSync(settingsPath, 'utf8'));
                
                // Logo dan favicon
                if (settings.design && settings.design.logoUrl && settings.design.logoUrl.startsWith('/uploads/')) {
                    usedImagePaths.add(settings.design.logoUrl.split('?')[0]);
                }
                
                if (settings.design && settings.design.faviconUrl && settings.design.faviconUrl.startsWith('/uploads/')) {
                    usedImagePaths.add(settings.design.faviconUrl.split('?')[0]);
                }
                
                // Background image jika ada
                if (settings.design && settings.design.backgroundImage && settings.design.backgroundImage.startsWith('/uploads/')) {
                    usedImagePaths.add(settings.design.backgroundImage.split('?')[0]);
                }
                
            } catch (err) {
                console.error('Error membaca file settings.json:', err);
            }
        }
        
        // 6. Periksa gambar yang digunakan di profil pengguna
        if (fs.existsSync(usersPath)) {
            try {
                const users = JSON.parse(fs.readFileSync(usersPath, 'utf8'));
                
                users.forEach(user => {
                    if (user.profileImage && user.profileImage.startsWith('/uploads/')) {
                        usedImagePaths.add(user.profileImage.split('?')[0]);
                    }
                });
                
            } catch (err) {
                console.error('Error membaca file users.json:', err);
            }
        }
        
        stats.usedFiles = usedImagePaths.size;
        console.log(`Jumlah file gambar yang digunakan: ${stats.usedFiles}`);
        
        // 7. Identifikasi file yang tidak digunakan
        for (const file of allFiles) {
            if (usedImagePaths.has(file.path)) {
                file.inUse = true;
            } else {
                stats.unusedFiles++;
            }
        }
        
        // 8. Hapus file yang tidak digunakan
        const unusedFiles = allFiles.filter(file => !file.inUse);
        console.log(`Jumlah file gambar yang tidak digunakan: ${unusedFiles.length}`);
        
        // Log detail file yang tidak digunakan
        unusedFiles.forEach(file => {
            console.log(`File tidak digunakan: ${file.filename}`);
        });
        
        // 9. Hapus file dari sistem
        for (const file of unusedFiles) {
            try {
                if (fs.existsSync(file.localPath)) {
                    // Hapus file
                    fs.unlinkSync(file.localPath);
                    stats.deletedFiles++;
                    console.log(`File berhasil dihapus: ${file.filename}`);
                }
            } catch (err) {
                console.error(`Error menghapus file ${file.filename}:`, err);
                stats.errors++;
            }
        }
        
        // 10. Update media.json
        if (fs.existsSync(mediaPath)) {
            try {
                let mediaData = JSON.parse(fs.readFileSync(mediaPath, 'utf8'));
                const beforeCount = mediaData.length;
                
                // Filter hanya file yang masih ada
                mediaData = mediaData.filter(item => {
                    // Cek jika file masih ada di sistem
                    const filePath = path.join(uploadsPath, item.filename);
                    return fs.existsSync(filePath);
                });
                
                // Hitung item yang dihapus
                stats.mediaJsonUpdated = beforeCount - mediaData.length;
                
                // Simpan kembali ke media.json
                fs.writeFileSync(mediaPath, JSON.stringify(mediaData, null, 2), 'utf8');
                console.log(`${stats.mediaJsonUpdated} item dihapus dari media.json`);
                
            } catch (err) {
                console.error('Error saat memperbarui media.json:', err);
                stats.errors++;
            }
        }
        
        const endTime = Date.now();
        const executionTime = (endTime - startTime) / 1000; // dalam detik
        
        return {
            success: true,
            executionTime,
            stats
        };
    } catch (error) {
        console.error('Error saat pembersihan media:', error);
        return {
            success: false,
            message: error.message || 'Terjadi kesalahan saat membersihkan media',
            error
        };
    }
}

// Test route untuk debugging
app.get('/test-route', (req, res) => {
    console.log('TEST ROUTE: accessed');
    res.send('Test route works!');
});

// Print all registered routes for debugging
console.log('==== REGISTERED ROUTES ====');
app._router.stack.forEach(function(r){
    if (r.route && r.route.path){
        console.log(`${Object.keys(r.route.methods)[0].toUpperCase()} ${r.route.path}`);
    }
});
console.log('==========================');

// Alternative cleanup route for testing
app.get('/admin/media-cleanup', checkAuth, checkRole(['admin']), (req, res) => {
    console.log('ALTERNATIVE ROUTE: /admin/media-cleanup accessed');
    res.render('admin/media-cleanup', {
        user: req.session.user,
        userRole: req.session.userRole,
        success_msg: req.flash('success_msg'),
        error_msg: req.flash('error_msg')
    });
});

// Alternative cleanup run route
app.post('/admin/media-cleanup/run', checkAuth, checkRole(['admin']), async (req, res) => {
    console.log('ALTERNATIVE ROUTE: /admin/media-cleanup/run accessed');
    try {
        // Jalankan pembersihan media
        const result = await cleanUnusedMedia();
        
        if (result.success) {
            req.flash('success_msg', `Pembersihan media berhasil! ${result.stats.deletedFiles} file dihapus dari ${result.stats.unusedFiles} file yang tidak digunakan. Waktu proses: ${result.executionTime.toFixed(2)} detik.`);
        } else {
            req.flash('error_msg', `Terjadi kesalahan: ${result.message}`);
        }
        
        return res.redirect('/admin/media-cleanup');
    } catch (error) {
        console.error('Error saat pembersihan media:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat membersihkan media');
        return res.redirect('/admin/media-cleanup');
    }
});

// Alternative cleanup API
app.post('/admin/api/media-cleanup', checkAuth, checkRole(['admin']), async (req, res) => {
    console.log('ALTERNATIVE ROUTE: /admin/api/media-cleanup accessed');
    try {
        // Jalankan pembersihan media
        const result = await cleanUnusedMedia();
        return res.json(result);
    } catch (error) {
        console.error('Error saat pembersihan media:', error);
        return res.status(500).json({
            success: false,
            message: error.message || 'Terjadi kesalahan saat membersihkan media',
            error: error.toString()
        });
    }
});

