/**
 * Middleware IP Whitelist
 * 
 * Middleware ini memeriksa apakah IP pengguna terdaftar dalam whitelist
 * untuk mengakses panel admin. Jika tidak, pengguna akan dialihkan ke halaman 404.
 */

const ipWhitelist = require('../config/ipWhitelist');

const checkIpWhitelist = (req, res, next) => {
    // Mendapatkan IP pengguna - improved for cPanel/proxy environments
    let clientIp;
    
    if (req.headers['x-forwarded-for']) {
        clientIp = req.headers['x-forwarded-for'].split(',')[0].trim();
    } else {
        clientIp = req.headers['x-real-ip'] || 
                  req.ip ||
                  req.connection.remoteAddress || 
                  req.socket.remoteAddress ||
                  (req.connection.socket ? req.connection.socket.remoteAddress : null);
    }
    
    // Menghapus IPv6 prefix jika ada
    const ipAddress = (clientIp || "").replace(/^::ffff:/, '');
    
    console.log('Memeriksa akses admin dari IP:', ipAddress);
    
    // Fungsi untuk memeriksa IP dengan wildcard pattern
    const ipMatchesPattern = (ip, pattern) => {
        // IPv6 localhost
        if (ip === "::1" && (pattern === "::1" || pattern === "localhost" || pattern === "127.0.0.1")) {
            return true;
        }
        
        // Jika pattern memiliki wildcard (*)
        if (pattern.includes('*')) {
            const patternParts = pattern.split('.');
            const ipParts = ip.split('.');
            
            // Memastikan jumlah bagian IP sama (IPv4)
            if (ipParts.length !== 4 || patternParts.length !== 4) {
                return false;
            }
            
            // Periksa setiap bagian dari IP
            for (let i = 0; i < 4; i++) {
                if (patternParts[i] !== '*' && patternParts[i] !== ipParts[i]) {
                    return false;
                }
            }
            
            return true;
        }
        
        // Jika tidak ada wildcard, periksa kecocokan langsung
        return ip === pattern;
    };
    
    // Periksa apakah IP ada dalam whitelist
    const ipAllowed = ipWhitelist.some(pattern => ipMatchesPattern(ipAddress, pattern));
    
    if (ipAllowed) {
        // IP diizinkan, lanjutkan ke middleware berikutnya
        return next();
    }
    
    // Jika IP tidak dalam whitelist, kirim ke halaman 404
    console.log('Akses admin ditolak untuk IP:', ipAddress);
    try {
        return res.status(404).render('404', {
            message: 'Halaman tidak ditemukan'
        });
    } catch (error) {
        console.error('Error rendering 404 page:', error);
        return res.status(404).send('Halaman tidak ditemukan');
    }
};

module.exports = checkIpWhitelist;