/**
 * Dynamic Content Handler
 * Script untuk menangani konten dinamis dari pengaturan admin
 */

document.addEventListener('DOMContentLoaded', function() {
    // Fungsi untuk memproses marquee text
    function processMarqueeText(text) {
        if (!text) return '';
        
        // Split berdasarkan ★ untuk mendapatkan item individual
        const items = text.split('★');
        let html = '';
        
        items.forEach((item, index) => {
            if (item.trim()) {
                html += `<span class="marquee-item">${item.trim()}</span>`;
                if (index < items.length - 1) {
                    html += `<span class="marquee-separator">★</span>`;
                }
            }
        });
        
        return html;
    }
    
    // Update marquee content jika ada
    const marqueeContainer = document.querySelector('.marquee-container marquee');
    if (marqueeContainer && window.siteSettings && window.siteSettings.design && window.siteSettings.design.marqueeText) {
        marqueeContainer.innerHTML = processMarqueeText(window.siteSettings.design.marqueeText);
    }
    
    // Update artikel title jika ada
    const artikelTitle = document.querySelector('.artikel-section h2');
    if (artikelTitle && window.siteSettings && window.siteSettings.design && window.siteSettings.design.artikelTitle) {
        artikelTitle.innerHTML = window.siteSettings.design.artikelTitle;
    }
    
    // Update artikel content jika ada
    const artikelContent = document.querySelector('.artikel-section article');
    if (artikelContent && window.siteSettings && window.siteSettings.design && window.siteSettings.design.artikelContent) {
        // Preserve h2 title
        const title = artikelContent.querySelector('h2').outerHTML;
        
        // Format artikel content
        let formattedContent = window.siteSettings.design.artikelContent;
        formattedContent = formattedContent.replace(/\n\n/g, '</p><p>');
        formattedContent = formattedContent.replace(/\n/g, '<br>');
        
        // Cek jika ada heading format (###)
        if (formattedContent.includes('###')) {
            formattedContent = formattedContent.replace(/###(.*?)###/g, '<h3 class="mt-4" style="color: var(--primary-color);">$1</h3>');
        }
        
        artikelContent.innerHTML = title + '<p>' + formattedContent + '</p>';
    }
    
    // Pastikan DOM telah sepenuhnya dirender sebelum memproses tombol CTA
    setTimeout(function() {
        // Debug log dari pengaturan tombol CTA
        console.log('CTA Settings:', {
            button1: {
                text: window.siteSettings?.design?.ctaButton1Text,
                url: window.siteSettings?.design?.ctaButton1Url,
                icon: window.siteSettings?.design?.ctaButton1Icon
            },
            button2: {
                text: window.siteSettings?.design?.ctaButton2Text,
                url: window.siteSettings?.design?.ctaButton2Url,
                icon: window.siteSettings?.design?.ctaButton2Icon
            }
        });

        // Update tombol CTA dengan manipulasi DOM langsung
        try {
            // Tombol 1
            updateCtaButton(
                '.main-title-section .row .col-6:first-child .cta-button',
                window.siteSettings?.design?.ctaButton1Url,
                window.siteSettings?.design?.ctaButton1Icon,
                window.siteSettings?.design?.ctaButton1Text
            );
            
            // Tombol 2
            updateCtaButton(
                '.main-title-section .row .col-6:last-child .cta-button',
                window.siteSettings?.design?.ctaButton2Url,
                window.siteSettings?.design?.ctaButton2Icon,
                window.siteSettings?.design?.ctaButton2Text
            );
        } catch (error) {
            console.error('Error updating CTA buttons:', error);
        }
    }, 100);
    
    // Fungsi untuk mengupdate tombol CTA
    function updateCtaButton(selector, url, icon, text) {
        const button = document.querySelector(selector);
        if (!button) {
            console.warn(`Button dengan selector ${selector} tidak ditemukan`);
            return;
        }
        
        console.log(`Updating button ${selector}:`, { url, icon, text });
        
        // Update URL
        if (url) {
            button.href = url;
        }
        
        // Update isi tombol (icon dan teks)
        if (icon || text) {
            const iconClass = icon || (selector.includes('first-child') ? 'fas fa-user-plus' : 'fas fa-comments');
            const buttonText = text || (selector.includes('first-child') ? 'DAFTAR SEKARANG' : 'HUBUNGI KAMI');
            
            button.innerHTML = `<i class="${iconClass} icon"></i>${buttonText}`;
        }
        
        console.log(`Button ${selector} setelah diupdate:`, button.outerHTML);
    }
}); 